# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class LicenseType(str, enum.Enum):
    MD = "MD"
    NP = "NP"
    PA = "PA"
    LMFT = "LMFT"
    LCPC = "LCPC"
    LCSW = "LCSW"
    PMHNP = "PMHNP"
    FNP = "FNP"
    LPCC = "LPCC"
    DO = "DO"
    RD = "RD"
    SLP = "SLP"
    APRN = "APRN"
    LPC = "LPC"
    PHD = "PHD"
    PSYD = "PSYD"
    LMSW = "LMSW"
    LMHC = "LMHC"
    OTHER_MASTERS = "OTHER_MASTERS"
    BCBA = "BCBA"
    UNKNOWN = "UNKNOWN"
    RPH = "RPH"
    PHT = "PHT"
    LAC = "LAC"
    LMT = "LMT"
    DC = "DC"
    ND = "ND"
    MA = "MA"
    PT = "PT"
    IBCLC = "IBCLC"
    RN = "RN"
    DPT = "DPT"
    LCMHC = "LCMHC"
    CNM = "CNM"
    RNFA = "RNFA"
    ACSW = "ACSW"
    APC = "APC"
    BCABA = "BCABA"
    BHA = "BHA"
    OD = "OD"
    DPM = "DPM"
    DA = "DA"
    DDS = "DDS"
    DEH = "DEH"
    DMD = "DMD"
    PTA = "PTA"
    LCADC = "LCADC"
    LCAT = "LCAT"
    LCMHCS = "LCMHCS"
    LCMHCA = "LCMHCA"
    LCSWA = "LCSWA"
    LICSW = "LICSW"
    LISW = "LISW"
    LMFTS = "LMFTS"
    LMFTA = "LMFTA"
    LPCI = "LPCI"
    LSCSW = "LSCSW"
    MHCA = "MHCA"
    MHT = "MHT"
    RBT = "RBT"
    RCSWI = "RCSWI"
    RHMCI = "RHMCI"
    LPN = "LPN"
    OTD = "OTD"
    OMS = "OMS"
    MFTA = "MFTA"
    APCC = "APCC"
    DNP = "DNP"

    def visit(
        self,
        md: typing.Callable[[], T_Result],
        np: typing.Callable[[], T_Result],
        pa: typing.Callable[[], T_Result],
        lmft: typing.Callable[[], T_Result],
        lcpc: typing.Callable[[], T_Result],
        lcsw: typing.Callable[[], T_Result],
        pmhnp: typing.Callable[[], T_Result],
        fnp: typing.Callable[[], T_Result],
        lpcc: typing.Callable[[], T_Result],
        do: typing.Callable[[], T_Result],
        rd: typing.Callable[[], T_Result],
        slp: typing.Callable[[], T_Result],
        aprn: typing.Callable[[], T_Result],
        lpc: typing.Callable[[], T_Result],
        phd: typing.Callable[[], T_Result],
        psyd: typing.Callable[[], T_Result],
        lmsw: typing.Callable[[], T_Result],
        lmhc: typing.Callable[[], T_Result],
        other_masters: typing.Callable[[], T_Result],
        bcba: typing.Callable[[], T_Result],
        unknown: typing.Callable[[], T_Result],
        rph: typing.Callable[[], T_Result],
        pht: typing.Callable[[], T_Result],
        lac: typing.Callable[[], T_Result],
        lmt: typing.Callable[[], T_Result],
        dc: typing.Callable[[], T_Result],
        nd: typing.Callable[[], T_Result],
        ma: typing.Callable[[], T_Result],
        pt: typing.Callable[[], T_Result],
        ibclc: typing.Callable[[], T_Result],
        rn: typing.Callable[[], T_Result],
        dpt: typing.Callable[[], T_Result],
        lcmhc: typing.Callable[[], T_Result],
        cnm: typing.Callable[[], T_Result],
        rnfa: typing.Callable[[], T_Result],
        acsw: typing.Callable[[], T_Result],
        apc: typing.Callable[[], T_Result],
        bcaba: typing.Callable[[], T_Result],
        bha: typing.Callable[[], T_Result],
        od: typing.Callable[[], T_Result],
        dpm: typing.Callable[[], T_Result],
        da: typing.Callable[[], T_Result],
        dds: typing.Callable[[], T_Result],
        deh: typing.Callable[[], T_Result],
        dmd: typing.Callable[[], T_Result],
        pta: typing.Callable[[], T_Result],
        lcadc: typing.Callable[[], T_Result],
        lcat: typing.Callable[[], T_Result],
        lcmhcs: typing.Callable[[], T_Result],
        lcmhca: typing.Callable[[], T_Result],
        lcswa: typing.Callable[[], T_Result],
        licsw: typing.Callable[[], T_Result],
        lisw: typing.Callable[[], T_Result],
        lmfts: typing.Callable[[], T_Result],
        lmfta: typing.Callable[[], T_Result],
        lpci: typing.Callable[[], T_Result],
        lscsw: typing.Callable[[], T_Result],
        mhca: typing.Callable[[], T_Result],
        mht: typing.Callable[[], T_Result],
        rbt: typing.Callable[[], T_Result],
        rcswi: typing.Callable[[], T_Result],
        rhmci: typing.Callable[[], T_Result],
        lpn: typing.Callable[[], T_Result],
        otd: typing.Callable[[], T_Result],
        oms: typing.Callable[[], T_Result],
        mfta: typing.Callable[[], T_Result],
        apcc: typing.Callable[[], T_Result],
        dnp: typing.Callable[[], T_Result],
    ) -> T_Result:
        if self is LicenseType.MD:
            return md()
        if self is LicenseType.NP:
            return np()
        if self is LicenseType.PA:
            return pa()
        if self is LicenseType.LMFT:
            return lmft()
        if self is LicenseType.LCPC:
            return lcpc()
        if self is LicenseType.LCSW:
            return lcsw()
        if self is LicenseType.PMHNP:
            return pmhnp()
        if self is LicenseType.FNP:
            return fnp()
        if self is LicenseType.LPCC:
            return lpcc()
        if self is LicenseType.DO:
            return do()
        if self is LicenseType.RD:
            return rd()
        if self is LicenseType.SLP:
            return slp()
        if self is LicenseType.APRN:
            return aprn()
        if self is LicenseType.LPC:
            return lpc()
        if self is LicenseType.PHD:
            return phd()
        if self is LicenseType.PSYD:
            return psyd()
        if self is LicenseType.LMSW:
            return lmsw()
        if self is LicenseType.LMHC:
            return lmhc()
        if self is LicenseType.OTHER_MASTERS:
            return other_masters()
        if self is LicenseType.BCBA:
            return bcba()
        if self is LicenseType.UNKNOWN:
            return unknown()
        if self is LicenseType.RPH:
            return rph()
        if self is LicenseType.PHT:
            return pht()
        if self is LicenseType.LAC:
            return lac()
        if self is LicenseType.LMT:
            return lmt()
        if self is LicenseType.DC:
            return dc()
        if self is LicenseType.ND:
            return nd()
        if self is LicenseType.MA:
            return ma()
        if self is LicenseType.PT:
            return pt()
        if self is LicenseType.IBCLC:
            return ibclc()
        if self is LicenseType.RN:
            return rn()
        if self is LicenseType.DPT:
            return dpt()
        if self is LicenseType.LCMHC:
            return lcmhc()
        if self is LicenseType.CNM:
            return cnm()
        if self is LicenseType.RNFA:
            return rnfa()
        if self is LicenseType.ACSW:
            return acsw()
        if self is LicenseType.APC:
            return apc()
        if self is LicenseType.BCABA:
            return bcaba()
        if self is LicenseType.BHA:
            return bha()
        if self is LicenseType.OD:
            return od()
        if self is LicenseType.DPM:
            return dpm()
        if self is LicenseType.DA:
            return da()
        if self is LicenseType.DDS:
            return dds()
        if self is LicenseType.DEH:
            return deh()
        if self is LicenseType.DMD:
            return dmd()
        if self is LicenseType.PTA:
            return pta()
        if self is LicenseType.LCADC:
            return lcadc()
        if self is LicenseType.LCAT:
            return lcat()
        if self is LicenseType.LCMHCS:
            return lcmhcs()
        if self is LicenseType.LCMHCA:
            return lcmhca()
        if self is LicenseType.LCSWA:
            return lcswa()
        if self is LicenseType.LICSW:
            return licsw()
        if self is LicenseType.LISW:
            return lisw()
        if self is LicenseType.LMFTS:
            return lmfts()
        if self is LicenseType.LMFTA:
            return lmfta()
        if self is LicenseType.LPCI:
            return lpci()
        if self is LicenseType.LSCSW:
            return lscsw()
        if self is LicenseType.MHCA:
            return mhca()
        if self is LicenseType.MHT:
            return mht()
        if self is LicenseType.RBT:
            return rbt()
        if self is LicenseType.RCSWI:
            return rcswi()
        if self is LicenseType.RHMCI:
            return rhmci()
        if self is LicenseType.LPN:
            return lpn()
        if self is LicenseType.OTD:
            return otd()
        if self is LicenseType.OMS:
            return oms()
        if self is LicenseType.MFTA:
            return mfta()
        if self is LicenseType.APCC:
            return apcc()
        if self is LicenseType.DNP:
            return dnp()



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ......core.datetime_utils import serialize_datetime
from .....commons.types.resource_page import ResourcePage
from .encounter import Encounter


class EncounterPage(ResourcePage):
    """
    import datetime
    import uuid

    from candid import (Claim, ClaimStatus, DateRangeOptionalEnd, Diagnosis,
                        DiagnosisTypeCode, EncounterServiceFacility, Era,
                        FacilityTypeCode, Gender, InsuranceTypeCode, Invoice,
                        InvoiceItem, InvoiceStatus, Patient,
                        PatientRelationshipToInsuredCodeAll, PhoneNumber,
                        PhoneNumberType, ProcedureModifier, ServiceLineUnits,
                        SourceOfPaymentCode, State, StreetAddressLongZip,
                        StreetAddressShortZip, Subscriber, Tag, TagColorEnum)
    from candid.resources.billing_notes.v_2 import BillingNote
    from candid.resources.encounter_providers.v_2 import EncounterProvider
    from candid.resources.encounters.v_4 import (BillableStatusType, ClinicalNote,
                                                 ClinicalNoteCategory,
                                                 CodingAttributionType, Encounter,
                                                 EncounterOwnerOfNextActionType,
                                                 EncounterPage,
                                                 EncounterSubmissionOriginType,
                                                 IntakeFollowUp, IntakeQuestion,
                                                 IntakeResponseAndFollowUps,
                                                 Intervention,
                                                 InterventionCategory, Lab,
                                                 LabCodeType, Medication,
                                                 NoteCategory,
                                                 PatientHistoryCategory,
                                                 PatientHistoryCategoryEnum,
                                                 ResponsiblePartyType,
                                                 ServiceAuthorizationExceptionCode,
                                                 SynchronicityType, Vitals)
    from candid.resources.guarantor.v_1 import Guarantor
    from candid.resources.insurance_cards.v_2 import InsuranceCard
    from candid.resources.patient_payments.v_3 import (PatientPayment,
                                                       PatientPaymentSource,
                                                       PatientPaymentStatus)
    from candid.resources.service_lines.v_2 import (DenialReasonContent,
                                                    ServiceLine,
                                                    ServiceLineAdjustment,
                                                    ServiceLineDenialReason,
                                                    ServiceLineEraData)

    EncounterPage(items=[Encounter(encounter_id=uuid.UUID("b2506e84-4470-4cba-8a68-6883367739e1", ), claims=[Claim(claim_id=uuid.UUID("dd9d7f82-37b5-449d-aa63-26925398335b", ), status=ClaimStatus.BILLER_RECEIVED, clearinghouse="Change Healthcare", clearinghouse_claim_id="5BA7C3AB-2BC2-496C-8B10-6CAC73D0729D", payer_claim_id="9BB9F259-9756-4F16-8F53-9DBB9F7EB1BB", service_lines=[ServiceLine(modifiers=[ProcedureModifier.22], charge_amount_cents=10000, allowed_amount_cents=8000, insurance_balance_cents=0, patient_balance_cents=2000, paid_amount_cents=8000, patient_responsibility_cents=2000, diagnosis_id_zero=uuid.UUID("4ac84bcd-12f5-4f86-a57b-e06749127c98", ), diagnosis_id_one=uuid.UUID("eea5ca5a-8b43-45fd-8cbd-c6cc1103e759", ), diagnosis_id_two=uuid.UUID("5c4aa029-2db9-4202-916e-e93c708f65ff", ), diagnosis_id_three=uuid.UUID("81795126-a3ac-443c-b47e-7259a16ab4a2", ), service_line_era_data=ServiceLineEraData(service_line_adjustments=[ServiceLineAdjustment(created_at=datetime.datetime.fromisoformat("2023-01-01 00:00:00+00:00", ), adjustment_group_code="CO", adjustment_reason_code="CO", adjustment_amount_cents=1000, adjustment_note="test_note", )], remittance_advice_remark_codes=["N362"], ), service_line_manual_adjustments=[ServiceLineAdjustment(created_at=datetime.datetime.fromisoformat("2023-01-01 00:00:00+00:00", ), adjustment_group_code="CO", adjustment_reason_code="CO", adjustment_amount_cents=1000, adjustment_note="test_note", )], related_invoices=[Invoice(id=uuid.UUID("901be2f1-41bc-456e-9987-4fe2f84f9d75", ), created_at=datetime.datetime.fromisoformat("2023-01-01 00:00:00+00:00", ), updated_at=datetime.datetime.fromisoformat("2023-01-01 00:00:00+00:00", ), organzation_id=uuid.UUID("f13f73d4-4344-46ea-9d93-33bcffbb9f36", ), source_id="9B626577-8808-4F28-9ED1-F0DFF0D49BBC", source_customer_id="624D1972-8C69-4C2F-AEFA-10856F734DB3", patient_external_id="10FED4D6-4C5A-48DF-838A-EEF45A74788D", note="test_note", due_date="2023-10-10", status=InvoiceStatus.DRAFT, url="https://example.com", customer_invoice_url="https://example.com", items=[InvoiceItem(service_line_id=uuid.UUID("ced00f23-6e68-4678-9dbc-f5aa2969a565", ), amount_cents=500, )], )], denial_reason=ServiceLineDenialReason(reason=DenialReasonContent.AUTHORIZATION_REQUIRED, ), place_of_service_code=FacilityTypeCode.01, service_line_id=uuid.UUID("ced00f23-6e68-4678-9dbc-f5aa2969a565", ), procedure_code="99213", quantity="1", units=ServiceLineUnits.MJ, claim_id=uuid.UUID("026a1fb8-748e-4859-a2d7-3ea9e07d25ae", ), date_of_service_range=DateRangeOptionalEnd(start_date="2023-01-01", end_date="2023-01-03", ), )], eras=[Era(era_id=uuid.UUID("4d844ef1-2253-43cd-a4f1-6db7e65cb54b", ), check_number="CHK12345", check_date="2023-10-12", )], )], patient=Patient(individual_id=uuid.UUID("93ddbebf-4956-4482-9a6c-21499b7e4e5d", ), phone_numbers=[PhoneNumber(number="1234567890", type=PhoneNumberType.HOME, )], phone_consent=True, email="johndoe@joincandidhealth.com", email_consent=True, external_id="49460F77-6456-41F1-AC6D-0AED08614D39", date_of_birth="2000-01-01", address=StreetAddressShortZip(address_1="123 Main St", address_2="Apt 1", city="New York", state=State.NY, zip_code="10001", zip_plus_four_code="1234", ), first_name="John", last_name="Doe", gender=Gender.MALE, ), guarantor=Guarantor(guarantor_id=uuid.UUID("8bbdbe63-58d3-4d40-98c9-40403c050977", ), phone_numbers=[PhoneNumber(number="1234567890", type=PhoneNumberType.HOME, )], phone_consent=True, email="johndoe@joincandidhealth.com", email_consent=True, first_name="John", last_name="Doe", external_id="49460F77-6456-41F1-AC6D-0AED08614D39", date_of_birth=datetime.date.fromisoformat("2000-01-01", ), address=StreetAddressShortZip(address_1="123 Main St", address_2="Apt 1", city="New York", state=State.NY, zip_code="10001", zip_plus_four_code="1234", ), ), billing_provider=EncounterProvider(provider_id=uuid.UUID("d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32", ), address=StreetAddressLongZip(address_1="123 Main St", address_2="Apt 1", city="New York", state=State.NY, zip_code="10001", zip_plus_four_code="1234", ), tax_id="123456789", npi="1234567890", taxonomy_code="207Q00000X", first_name="John", last_name="Doe", organization_name="Organization Name", ), rendering_provider=EncounterProvider(provider_id=uuid.UUID("d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32", ), address=StreetAddressLongZip(address_1="123 Main St", address_2="Apt 1", city="New York", state=State.NY, zip_code="10001", zip_plus_four_code="1234", ), tax_id="123456789", npi="1234567890", taxonomy_code="207Q00000X", first_name="John", last_name="Doe", organization_name="Organization Name", ), referring_provider=EncounterProvider(provider_id=uuid.UUID("d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32", ), address=StreetAddressLongZip(address_1="123 Main St", address_2="Apt 1", city="New York", state=State.NY, zip_code="10001", zip_plus_four_code="1234", ), tax_id="123456789", npi="1234567890", taxonomy_code="207Q00000X", first_name="John", last_name="Doe", organization_name="Organization Name", ), service_facility=EncounterServiceFacility(service_facility_id=uuid.UUID("2861487b-232c-4ded-a874-616a5db0688f", ), organization_name="Test Organization", address=StreetAddressLongZip(address_1="123 Main St", address_2="Apt 1", city="New York", state=State.NY, zip_code="10001", zip_plus_four_code="1234", ), ), subscriber_primary=Subscriber(individual_id=uuid.UUID("797348a9-e7e8-4e59-8628-95390d079c0b", ), insurance_card=InsuranceCard(insurance_card_id=uuid.UUID("ca5b7711-4419-4161-9b7c-3494ac40c8d4", ), member_id="E85313B4-0FFC-4119-8042-8161A4ECFF0A", payer_name="John Doe", payer_id="836DDAA6-863F-4020-ACCA-205A689F0002", rx_bin="610014", rx_pcn="MEDDPRIME", image_url_front="https://s3.amazonaws.com/front.jpg", image_url_back="https://s3.amazonaws.com/back.jpg", group_number="ABC12345", plan_name="Silver PPO Plan", plan_type=SourceOfPaymentCode.09, insurance_type=InsuranceTypeCode.12, ), patient_relationship_to_subscriber_code=PatientRelationshipToInsuredCodeAll.01, date_of_birth="2000-01-01", address=StreetAddressShortZip(address_1="123 Main St", address_2="Apt 1", city="New York", state=State.NY, zip_code="10001", zip_plus_four_code="1234", ), first_name="John", last_name="Doe", gender=Gender.MALE, ), subscriber_secondary=Subscriber(individual_id=uuid.UUID("797348a9-e7e8-4e59-8628-95390d079c0b", ), insurance_card=InsuranceCard(insurance_card_id=uuid.UUID("ca5b7711-4419-4161-9b7c-3494ac40c8d4", ), member_id="E85313B4-0FFC-4119-8042-8161A4ECFF0A", payer_name="John Doe", payer_id="836DDAA6-863F-4020-ACCA-205A689F0002", rx_bin="610014", rx_pcn="MEDDPRIME", image_url_front="https://s3.amazonaws.com/front.jpg", image_url_back="https://s3.amazonaws.com/back.jpg", group_number="ABC12345", plan_name="Silver PPO Plan", plan_type=SourceOfPaymentCode.09, insurance_type=InsuranceTypeCode.12, ), patient_relationship_to_subscriber_code=PatientRelationshipToInsuredCodeAll.01, date_of_birth="2000-01-01", address=StreetAddressShortZip(address_1="123 Main St", address_2="Apt 1", city="New York", state=State.NY, zip_code="10001", zip_plus_four_code="1234", ), first_name="John", last_name="Doe", gender=Gender.MALE, ), url="https://example.com", diagnoses=[Diagnosis(diagnosis_id=uuid.UUID("5c770e00-4bbf-42af-a73f-99c5e91fc0db", ), created_at=datetime.datetime.fromisoformat("2023-01-01 00:00:00+00:00", ), updated_at=datetime.datetime.fromisoformat("2023-01-01 00:00:00+00:00", ), encounter_id=uuid.UUID("3f63985b-51a4-4dd4-9418-7d50b2520792", ), name="John Doe", code_type=DiagnosisTypeCode.ABF, code="I10", )], clinical_notes=[ClinicalNoteCategory(category=NoteCategory.CLINICAL, notes=["Patient complained of mild chest pain."], notes_structured=[ClinicalNote(text="Mild chest pain since morning.", author_name="John Doe", author_npi="1234567890", timestamp=datetime.datetime.fromisoformat("2023-01-01 00:00:00+00:00", ), )], )], billing_notes=[BillingNote(billing_note_id=uuid.UUID("99882eea-936f-4e71-bc4f-520e4d14e3e2", ), encounter_id=uuid.UUID("8bcfb6a8-2876-4111-9e3f-602b541fcf62", ), created_at=datetime.datetime.fromisoformat("2023-01-01 00:00:00+00:00", ), author_auth_0_id="F0DE3BF9-F9A1-4FA7-BF6B-28C0B46BADD8", author_name="John Doe", text="Patient was billed for an MRI.", )], place_of_service_code=FacilityTypeCode.01, place_of_service_code_as_submitted=FacilityTypeCode.01, patient_histories=[PatientHistoryCategory(category=PatientHistoryCategoryEnum.PRESENT_ILLNESS, questions=[IntakeQuestion(id="6E7FBCE4-A8EA-46D0-A8D8-FF83CA3BB176", text="Do you have any allergies?", responses=[IntakeResponseAndFollowUps(response="No allergies", follow_ups=[IntakeFollowUp(id="4F3D57F9-AC94-49D6-87E4-E804B709917A", text="Do you have any allergies?", response="No allergies", )], )], )], )], patient_payments=[PatientPayment(patient_payment_id="CF237BE1-E793-4BBF-8958-61D5179D1D0D", organization_id=uuid.UUID("0788ca2a-b20d-4b8e-b8d4-07fa0b3b4907", ), source_internal_id="D1A76039-D5C5-4323-A2FC-B7C8B6AEF6A4", source=PatientPaymentSource.MANUAL_ENTRY, amount_cents=2000, payment_timestamp=datetime.datetime.fromisoformat("2023-01-01 00:00:00+00:00", ), status=PatientPaymentStatus.PENDING, payment_name="John Doe", payment_note="test payment note", patient_external_id="B7437260-D6B4-48CF-B9D7-753C09F34E76", encounter_external_id="0F26B9C3-199F-4CBB-A166-B87EA7C631BB", service_line_id=uuid.UUID("b557dc86-c629-478c-850a-02d45ac11783", ), )], tags=[Tag(creator_id="00EB5A46-35C6-441B-9751-AF307AEF5888", tag_id="void-claim-submitted", description="to indicate claims where a void claim has been submitted", color=TagColorEnum.BLACK, )], coding_attribution=CodingAttributionType.CANDID, work_queue_id="000856FE-1024-418F-BF96-2E7347AB4520", work_queue_membership_activated_at=datetime.datetime.fromisoformat("2023-01-01 00:00:00+00:00", ), owner_of_next_action=EncounterOwnerOfNextActionType.CANDID, submission_origin=EncounterSubmissionOriginType.CANDID, external_id="5C21490F-A9C0-45F4-B5DB-136E3AEC617A", date_of_service="2023-01-01", end_date_of_service="2023-01-05", prior_authorization_number="PA1234567", patient_authorized_release=True, benefits_assigned_to_provider=True, provider_accepts_assignment=True, appointment_type="Routine Checkup", existing_medications=[Medication(name="Lisinopril", rx_cui="860975", dosage="10mg", dosage_form="Tablet", frequency="Once Daily", as_needed=True, )], vitals=Vitals(height_in=70, weight_lbs=165, blood_pressure_systolic_mmhg=115, blood_pressure_diastolic_mmhg=85, body_temperature_f=98.0, ), interventions=[Intervention(name="Physical Therapy Session", category=InterventionCategory.LIFESTYLE, description="A session focused on improving muscular strength, flexibility, and range of motion post-injury.", medication=Medication(name="Lisinopril", rx_cui="860975", dosage="10mg", dosage_form="Tablet", frequency="Once Daily", as_needed=True, ), labs=[Lab(name="Genetic Health Labs", code="GH12345", code_type=LabCodeType.QUEST, )], )], pay_to_address=StreetAddressLongZip(address_1="123 Main St", address_2="Apt 1", city="New York", state=State.NY, zip_code="10001", zip_plus_four_code="1234", ), synchronicity=SynchronicityType.SYNCHRONOUS, billable_status=BillableStatusType.BILLABLE, responsible_party=ResponsiblePartyType.INSURANCE_PAY, service_authorization_exception_code=ServiceAuthorizationExceptionCode.1, admission_date="2023-01-01", discharge_date="2023-01-05", onset_of_current_illness_or_symptom_date="2023-01-01", )], )
    """

    items: typing.List[Encounter]

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ......core.datetime_utils import serialize_datetime
from .....commons.types.street_address_long_zip import StreetAddressLongZip
from .encounter_provider_base import EncounterProviderBase

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class ReferringProvider(EncounterProviderBase):
    npi: str = pydantic.Field(
        description=(
            "A National Provider Identifier is a unique 10-digit identification\n"
            "number issued to health care providers in the United States\n"
        )
    )
    taxonomy_code: typing.Optional[str] = None
    address: typing.Optional[StreetAddressLongZip] = None

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

import typing_extensions

from ...commons.types.claim_id import ClaimId
from ...commons.types.provider_id import ProviderId
from ...commons.types.service_line_id import ServiceLineId

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class AllocationTargetCreate_ServiceLineById(pydantic.BaseModel):
    type: typing_extensions.Literal["service_line_by_id"]
    value: ServiceLineId

    class Config:
        frozen = True
        smart_union = True


class AllocationTargetCreate_ClaimById(pydantic.BaseModel):
    type: typing_extensions.Literal["claim_by_id"]
    value: ClaimId

    class Config:
        frozen = True
        smart_union = True


class AllocationTargetCreate_BillingProviderById(pydantic.BaseModel):
    type: typing_extensions.Literal["billing_provider_by_id"]
    value: ProviderId

    class Config:
        frozen = True
        smart_union = True


class AllocationTargetCreate_Unattributed(pydantic.BaseModel):
    type: typing_extensions.Literal["unattributed"]

    class Config:
        frozen = True
        smart_union = True


AllocationTargetCreate = typing.Union[
    AllocationTargetCreate_ServiceLineById,
    AllocationTargetCreate_ClaimById,
    AllocationTargetCreate_BillingProviderById,
    AllocationTargetCreate_Unattributed,
]



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ......core.datetime_utils import serialize_datetime
from .intervention_category import InterventionCategory
from .lab import Lab
from .medication import Medication

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class Intervention(pydantic.BaseModel):
    """
    from candid.resources.encounters.v_4 import (
        Intervention,
        InterventionCategory,
        Lab,
        LabCodeType,
        Medication,
    )

    Intervention(
        name="Physical Therapy Session",
        category=InterventionCategory.LIFESTYLE,
        description="A session focused on improving muscular strength, flexibility, and range of motion post-injury.",
        medication=Medication(
            name="Lisinopril",
            rx_cui="860975",
            dosage="10mg",
            dosage_form="Tablet",
            frequency="Once Daily",
            as_needed=True,
        ),
        labs=[
            Lab(
                name="Genetic Health Labs",
                code="GH12345",
                code_type=LabCodeType.QUEST,
            )
        ],
    )
    """

    name: str
    category: InterventionCategory
    description: typing.Optional[str] = pydantic.Field(
        default=None,
        description="\"Examples: 'Birth Control LAC', 'Tracking', 'Stress Management', 'Supplement', 'Labs'\"",
    )
    medication: typing.Optional[Medication] = pydantic.Field(
        default=None, description="Required when `type` is `allopathic`."
    )
    labs: typing.Optional[typing.List[Lab]] = pydantic.Field(
        default=None, description="Required when `type` is `tests`."
    )

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ......core.datetime_utils import serialize_datetime
from .....commons.types.street_address_long_zip import StreetAddressLongZip
from .service_facility_mode import ServiceFacilityMode
from .service_facility_operational_status import ServiceFacilityOperationalStatus
from .service_facility_physical_type import ServiceFacilityPhysicalType
from .service_facility_status import ServiceFacilityStatus
from .service_facility_type import ServiceFacilityType

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class OrganizationServiceFacilityUpdate(pydantic.BaseModel):
    """
    from candid import State, StreetAddressLongZip
    from candid.resources.organization_service_facilities.v_2 import (
        OrganizationServiceFacilityUpdate,
        ServiceFacilityMode,
        ServiceFacilityOperationalStatus,
        ServiceFacilityPhysicalType,
        ServiceFacilityStatus,
        ServiceFacilityType,
    )

    OrganizationServiceFacilityUpdate(
        name="Test Service Facility",
        aliases=["Test Service Facility Alias"],
        description="Test Service Facility Description",
        status=ServiceFacilityStatus.ACTIVE,
        operational_status=ServiceFacilityOperationalStatus.C,
        mode=ServiceFacilityMode.INSTANCE,
        type=ServiceFacilityType.DX,
        physical_type=ServiceFacilityPhysicalType.SI,
        telecoms=["555-555-5555"],
        address=StreetAddressLongZip(
            address_1="123 Main St",
            address_2="Apt 1",
            city="New York",
            state=State.NY,
            zip_code="10001",
            zip_plus_four_code="1234",
        ),
    )
    """

    name: typing.Optional[str] = pydantic.Field(default=None, description="The name of the service facility.")
    aliases: typing.Optional[typing.List[str]] = pydantic.Field(
        default=None, description="A list of alternate names for the service facility."
    )
    description: typing.Optional[str] = pydantic.Field(
        default=None, description="A description of the service facility."
    )
    npi: typing.Optional[str] = pydantic.Field(
        default=None,
        description=(
            "An NPI specific to the service facility if applicable, i.e. if it has one and is not under the billing provider's NPI.\n"
            "Box 32 section (a) of the CMS-1500 claim form.\n"
        ),
    )
    status: typing.Optional[ServiceFacilityStatus] = pydantic.Field(
        default=None, description="The status of the service facility."
    )
    operational_status: typing.Optional[ServiceFacilityOperationalStatus] = pydantic.Field(
        default=None, description="The operational status of the service facility."
    )
    mode: typing.Optional[ServiceFacilityMode] = pydantic.Field(
        default=None, description="The mode of the service facility."
    )
    type: typing.Optional[ServiceFacilityType] = pydantic.Field(
        default=None, description="The type of the service facility."
    )
    physical_type: typing.Optional[ServiceFacilityPhysicalType] = pydantic.Field(
        default=None, description="The physical type of the service facility."
    )
    telecoms: typing.Optional[typing.List[str]] = pydantic.Field(
        default=None, description="A list of contact methods for the service facility."
    )
    address: typing.Optional[StreetAddressLongZip] = pydantic.Field(
        default=None, description="The address of the service facility."
    )

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ......core.datetime_utils import serialize_datetime
from .....commons.types.date import Date
from .employment_status import EmploymentStatus
from .organization_provider_base import OrganizationProviderBase
from .organization_provider_id import OrganizationProviderId

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class OrganizationProvider(OrganizationProviderBase):
    organization_provider_id: OrganizationProviderId = pydantic.Field(description="Auto-generated ID set on creation")
    employment_status: EmploymentStatus = pydantic.Field(description="The employment status for the provider.")
    employment_start_date: typing.Optional[Date] = pydantic.Field(
        default=None, description="The employment start date for the provider."
    )
    employment_termination_date: typing.Optional[Date] = pydantic.Field(
        default=None, description="The employment termination date for the provider."
    )

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

from .account_type import AccountType
from .allocation import Allocation
from .allocation_create import AllocationCreate
from .allocation_target import (
    AllocationTarget,
    AllocationTarget_BillingProviderId,
    AllocationTarget_Claim,
    AllocationTarget_ServiceLine,
    AllocationTarget_Unattributed,
)
from .allocation_target_create import (
    AllocationTargetCreate,
    AllocationTargetCreate_BillingProviderById,
    AllocationTargetCreate_ClaimById,
    AllocationTargetCreate_ServiceLineById,
    AllocationTargetCreate_Unattributed,
)
from .billing_provider_allocation_target import BillingProviderAllocationTarget
from .claim_allocation_target import ClaimAllocationTarget
from .invoice_update import InvoiceUpdate, InvoiceUpdate_Remove, InvoiceUpdate_Set
from .note_update import NoteUpdate, NoteUpdate_Remove, NoteUpdate_Set
from .patient_transaction_source import PatientTransactionSource
from .refund_reason import RefundReason
from .refund_reason_update import RefundReasonUpdate, RefundReasonUpdate_Remove, RefundReasonUpdate_Set
from .service_line_allocation_target import ServiceLineAllocationTarget

__all__ = [
    "AccountType",
    "Allocation",
    "AllocationCreate",
    "AllocationTarget",
    "AllocationTargetCreate",
    "AllocationTargetCreate_BillingProviderById",
    "AllocationTargetCreate_ClaimById",
    "AllocationTargetCreate_ServiceLineById",
    "AllocationTargetCreate_Unattributed",
    "AllocationTarget_BillingProviderId",
    "AllocationTarget_Claim",
    "AllocationTarget_ServiceLine",
    "AllocationTarget_Unattributed",
    "BillingProviderAllocationTarget",
    "ClaimAllocationTarget",
    "InvoiceUpdate",
    "InvoiceUpdate_Remove",
    "InvoiceUpdate_Set",
    "NoteUpdate",
    "NoteUpdate_Remove",
    "NoteUpdate_Set",
    "PatientTransactionSource",
    "RefundReason",
    "RefundReasonUpdate",
    "RefundReasonUpdate_Remove",
    "RefundReasonUpdate_Set",
    "ServiceLineAllocationTarget",
]



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

import typing_extensions

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class TaskActionExecutionMethod_CloseTask(pydantic.BaseModel):
    type: typing_extensions.Literal["close_task"]

    class Config:
        frozen = True
        smart_union = True


TaskActionExecutionMethod = typing.Union[TaskActionExecutionMethod_CloseTask]



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ......core.datetime_utils import serialize_datetime
from .....commons.types.facility_type_code import FacilityTypeCode
from .....commons.types.street_address_short_zip import StreetAddressShortZip
from .....organization_providers.resources.v_2.types.organization_provider_id import OrganizationProviderId
from .....organization_service_facilities.resources.v_2.types.organization_service_facility_id import (
    OrganizationServiceFacilityId,
)
from .expected_network_status_subscriber_information import ExpectedNetworkStatusSubscriberInformation
from .service_type import ServiceType

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class ComputeAllInNetworkProvidersRequest(pydantic.BaseModel):
    service_type: ServiceType = pydantic.Field(
        description=(
            "For some payers, payer routing depends on whether the rendered service qualifies as a behavioral health visit\n"
            "(e.g. Blue Shield of California routes to Magellan for behavioral health visits).\n"
            "\n"
            "For post appointment payer routing, Candid uses a CPT code list to determine whether the appointment qualifies as a\n"
            "behavioral health visit\n"
            "(see “Inputs: Service Type” in the appendix for list of qualifying CPT codes and behavioral health routing logic).\n"
            "Since CPT codes are not available pre-appointment, the service type input is used to determine whether the appointment is expected\n"
            "to qualify as behavioral health.\n"
        )
    )
    place_of_service_code: FacilityTypeCode = pydantic.Field(description="Expected place of service")
    subscriber_information: ExpectedNetworkStatusSubscriberInformation = pydantic.Field(
        description="Information present on the patient's insurance card"
    )
    patient_address: StreetAddressShortZip
    billing_provider_id: OrganizationProviderId
    organization_service_facility_id: OrganizationServiceFacilityId = pydantic.Field(
        description="The id of the service facility where the appointment will be rendered"
    )
    date_of_service: dt.date = pydantic.Field(description="Expected date of service")

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

import typing_extensions

from .....commons.types.claim_id import ClaimId
from .....commons.types.provider_id import ProviderId
from .....commons.types.service_line_id import ServiceLineId

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class InsuranceWriteOffTarget_ServiceLineId(pydantic.BaseModel):
    type: typing_extensions.Literal["service_line_id"]
    value: ServiceLineId

    class Config:
        frozen = True
        smart_union = True


class InsuranceWriteOffTarget_ClaimId(pydantic.BaseModel):
    type: typing_extensions.Literal["claim_id"]
    value: ClaimId

    class Config:
        frozen = True
        smart_union = True


class InsuranceWriteOffTarget_BillingProviderId(pydantic.BaseModel):
    type: typing_extensions.Literal["billing_provider_id"]
    value: ProviderId

    class Config:
        frozen = True
        smart_union = True


InsuranceWriteOffTarget = typing.Union[
    InsuranceWriteOffTarget_ServiceLineId, InsuranceWriteOffTarget_ClaimId, InsuranceWriteOffTarget_BillingProviderId
]



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class ServiceFacilityOperationalStatus(str, enum.Enum):
    """
    from candid.resources.organization_service_facilities.v_2 import (
        ServiceFacilityOperationalStatus,
    )

    ServiceFacilityOperationalStatus.C
    """

    CLOSED = "C"
    """
    The location is not operational and is closed.
    """

    HOUSEKEEPING = "H"
    """
    The location is currently undergoing cleaning or maintenance.
    """

    ISOLATED = "I"
    """
    The location is quarantined or isolated for special cases.
    """

    CONTAMINATED = "K"
    """
    The location has been identified as contaminated and is not currently usable.
    """

    OCCUPIED = "O"
    """
    The location is currently in use or occupied.
    """

    UNOCCUPIED = "U"
    """
    The location is operational but currently empty or not in use.
    """

    def visit(
        self,
        closed: typing.Callable[[], T_Result],
        housekeeping: typing.Callable[[], T_Result],
        isolated: typing.Callable[[], T_Result],
        contaminated: typing.Callable[[], T_Result],
        occupied: typing.Callable[[], T_Result],
        unoccupied: typing.Callable[[], T_Result],
    ) -> T_Result:
        if self is ServiceFacilityOperationalStatus.CLOSED:
            return closed()
        if self is ServiceFacilityOperationalStatus.HOUSEKEEPING:
            return housekeeping()
        if self is ServiceFacilityOperationalStatus.ISOLATED:
            return isolated()
        if self is ServiceFacilityOperationalStatus.CONTAMINATED:
            return contaminated()
        if self is ServiceFacilityOperationalStatus.OCCUPIED:
            return occupied()
        if self is ServiceFacilityOperationalStatus.UNOCCUPIED:
            return unoccupied()



# This file was auto-generated by Fern from our API Definition.

from .insurance_card import InsuranceCard
from .insurance_card_base import InsuranceCardBase
from .insurance_card_create import InsuranceCardCreate
from .insurance_card_id import InsuranceCardId

__all__ = ["InsuranceCard", "InsuranceCardBase", "InsuranceCardCreate", "InsuranceCardId"]



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ....core.datetime_utils import serialize_datetime
from .street_address_base import StreetAddressBase

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class StreetAddressLongZip(StreetAddressBase):
    """
    from candid import State, StreetAddressLongZip

    StreetAddressLongZip(
        address_1="123 Main St",
        address_2="Apt 1",
        city="New York",
        state=State.NY,
        zip_code="10001",
        zip_plus_four_code="1234",
    )
    """

    zip_plus_four_code: str = pydantic.Field(
        description="4-digit zip add-on code https://en.wikipedia.org/wiki/ZIP_Code#ZIP+4"
    )

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing
import urllib.parse
from json.decoder import JSONDecodeError

from .....core.api_error import ApiError
from .....core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from .....core.datetime_utils import serialize_datetime
from .....core.jsonable_encoder import jsonable_encoder
from .....core.remove_none_from_dict import remove_none_from_dict
from ....billing_notes.resources.v_2.types.billing_note_base import BillingNoteBase
from ....claim_submission.resources.v_1.types.external_claim_submission_create import ExternalClaimSubmissionCreate
from ....claims.types.claim_status import ClaimStatus
from ....commons.errors.entity_not_found_error import EntityNotFoundError
from ....commons.errors.http_request_validations_error import HttpRequestValidationsError
from ....commons.errors.unauthorized_error import UnauthorizedError
from ....commons.types.date import Date
from ....commons.types.delay_reason_code import DelayReasonCode
from ....commons.types.encounter_external_id import EncounterExternalId
from ....commons.types.encounter_id import EncounterId
from ....commons.types.entity_not_found_error_message import EntityNotFoundErrorMessage
from ....commons.types.facility_type_code import FacilityTypeCode
from ....commons.types.page_token import PageToken
from ....commons.types.request_validation_error import RequestValidationError
from ....commons.types.street_address_long_zip import StreetAddressLongZip
from ....commons.types.unauthorized_error_message import UnauthorizedErrorMessage
from ....commons.types.work_queue_id import WorkQueueId
from ....diagnoses.types.diagnosis_create import DiagnosisCreate
from ....diagnoses.types.diagnosis_id import DiagnosisId
from ....encounter_providers.resources.v_2.types.billing_provider import BillingProvider
from ....encounter_providers.resources.v_2.types.referring_provider import ReferringProvider
from ....encounter_providers.resources.v_2.types.rendering_provider import RenderingProvider
from ....guarantor.resources.v_1.types.guarantor_create import GuarantorCreate
from ....individual.types.patient_create import PatientCreate
from ....individual.types.subscriber_create import SubscriberCreate
from ....service_facility.types.encounter_service_facility_base import EncounterServiceFacilityBase
from ....service_lines.resources.v_2.types.service_line_create import ServiceLineCreate
from ....tags.types.tag_id import TagId
from .errors.cash_pay_payer_error import CashPayPayerError
from .errors.encounter_external_id_uniqueness_error import EncounterExternalIdUniquenessError
from .errors.encounter_guarantor_missing_contact_info_error import EncounterGuarantorMissingContactInfoError
from .errors.encounter_patient_control_number_uniqueness_error import EncounterPatientControlNumberUniquenessError
from .types.billable_status_type import BillableStatusType
from .types.cash_pay_payer_error_message import CashPayPayerErrorMessage
from .types.clinical_note_category_create import ClinicalNoteCategoryCreate
from .types.encounter import Encounter
from .types.encounter_external_id_uniqueness_error_type import EncounterExternalIdUniquenessErrorType
from .types.encounter_guarantor_missing_contact_info_error_type import EncounterGuarantorMissingContactInfoErrorType
from .types.encounter_owner_of_next_action_type import EncounterOwnerOfNextActionType
from .types.encounter_page import EncounterPage
from .types.encounter_patient_control_number_uniqueness_error_type import (
    EncounterPatientControlNumberUniquenessErrorType,
)
from .types.encounter_sort_options import EncounterSortOptions
from .types.intervention import Intervention
from .types.medication import Medication
from .types.patient_history_category import PatientHistoryCategory
from .types.prior_authorization_number import PriorAuthorizationNumber
from .types.responsible_party_type import ResponsiblePartyType
from .types.service_authorization_exception_code import ServiceAuthorizationExceptionCode
from .types.synchronicity_type import SynchronicityType
from .types.vitals import Vitals

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class V4Client:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def get_all(
        self,
        *,
        limit: typing.Optional[int] = None,
        claim_status: typing.Optional[ClaimStatus] = None,
        sort: typing.Optional[EncounterSortOptions] = None,
        page_token: typing.Optional[PageToken] = None,
        date_of_service_min: typing.Optional[Date] = None,
        date_of_service_max: typing.Optional[Date] = None,
        primary_payer_names: typing.Optional[str] = None,
        search_term: typing.Optional[str] = None,
        external_id: typing.Optional[EncounterExternalId] = None,
        diagnoses_updated_since: typing.Optional[dt.datetime] = None,
        tag_ids: typing.Optional[typing.Union[TagId, typing.List[TagId]]] = None,
        work_queue_id: typing.Optional[WorkQueueId] = None,
        billable_status: typing.Optional[BillableStatusType] = None,
        responsible_party: typing.Optional[ResponsiblePartyType] = None,
        owner_of_next_action: typing.Optional[EncounterOwnerOfNextActionType] = None,
    ) -> EncounterPage:
        """
        Parameters:
            - limit: typing.Optional[int]. Maximum number of entities per page, defaults to 100.

            - claim_status: typing.Optional[ClaimStatus]. Indicates the current status of an insurance claim within the billing process.

            - sort: typing.Optional[EncounterSortOptions]. Defaults to created_at:desc.

            - page_token: typing.Optional[PageToken].

            - date_of_service_min: typing.Optional[Date]. Date formatted as YYYY-MM-DD; eg: 2019-08-25.

            - date_of_service_max: typing.Optional[Date]. Date formatted as YYYY-MM-DD; eg: 2019-08-25.

            - primary_payer_names: typing.Optional[str]. Comma delimited string.

            - search_term: typing.Optional[str]. Filter by any of the following fields: encounter_id, claim_id, patient external_id,
                                                 patient date of birth, patient first name, patient last name,
                                                 or encounter external id.
            - external_id: typing.Optional[EncounterExternalId]. Filter to an exact match on encounter external_id, if one exists.

            - diagnoses_updated_since: typing.Optional[dt.datetime]. ISO 8601 timestamp; ideally in UTC (although not required): 2019-08-24T14:15:22Z.

            - tag_ids: typing.Optional[typing.Union[TagId, typing.List[TagId]]]. Filter by name of tags on encounters.

            - work_queue_id: typing.Optional[WorkQueueId].

            - billable_status: typing.Optional[BillableStatusType]. Defines if the Encounter is to be billed by Candid to the responsible_party. Examples for when this should be set to NOT_BILLABLE include if the Encounter has not occurred yet or if there is no intention of ever billing the responsible_party.

            - responsible_party: typing.Optional[ResponsiblePartyType]. Defines the party to be billed with the initial balance owed on the claim. Use SELF_PAY if you intend to bill self pay/cash pay.

            - owner_of_next_action: typing.Optional[EncounterOwnerOfNextActionType]. The party who is responsible for taking the next action on an Encounter, as defined by ownership of open Tasks.
        ---
        import datetime

        from candid import ClaimStatus
        from candid.client import CandidApi
        from candid.resources.encounters.v_4 import EncounterSortOptions

        client = CandidApi(
            token="YOUR_TOKEN",
        )
        client.encounters.v_4.get_all(
            limit=100,
            claim_status=ClaimStatus.BILLER_RECEIVED,
            sort=EncounterSortOptions.CREATED_AT_ASC,
            page_token="eyJ0b2tlbiI6IjEiLCJwYWdlX3Rva2VuIjoiMiJ9",
            date_of_service_min="2019-08-24",
            date_of_service_max="2019-08-25",
            primary_payer_names="Medicare,Medicaid",
            search_term="doe",
            external_id="123456",
            diagnoses_updated_since=datetime.datetime.fromisoformat(
                "2019-08-24 14:15:22+00:00",
            ),
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "api/encounters/v4"),
            params=remove_none_from_dict(
                {
                    "limit": limit,
                    "claim_status": claim_status,
                    "sort": sort,
                    "page_token": page_token,
                    "date_of_service_min": date_of_service_min,
                    "date_of_service_max": date_of_service_max,
                    "primary_payer_names": primary_payer_names,
                    "search_term": search_term,
                    "external_id": external_id,
                    "diagnoses_updated_since": serialize_datetime(diagnoses_updated_since)
                    if diagnoses_updated_since is not None
                    else None,
                    "tag_ids": tag_ids,
                    "work_queue_id": work_queue_id,
                    "billable_status": billable_status,
                    "responsible_party": responsible_party,
                    "owner_of_next_action": owner_of_next_action,
                }
            ),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(EncounterPage, _response_json)  # type: ignore
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get(self, encounter_id: EncounterId) -> Encounter:
        """
        Parameters:
            - encounter_id: EncounterId.
        """
        _response = self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", f"api/encounters/v4/{encounter_id}"),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(Encounter, _response_json)  # type: ignore
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def create(
        self,
        *,
        patient: PatientCreate,
        billing_provider: BillingProvider,
        rendering_provider: RenderingProvider,
        referring_provider: typing.Optional[ReferringProvider] = OMIT,
        service_facility: typing.Optional[EncounterServiceFacilityBase] = OMIT,
        subscriber_primary: typing.Optional[SubscriberCreate] = OMIT,
        subscriber_secondary: typing.Optional[SubscriberCreate] = OMIT,
        diagnoses: typing.List[DiagnosisCreate],
        clinical_notes: typing.Optional[typing.List[ClinicalNoteCategoryCreate]] = OMIT,
        billing_notes: typing.Optional[typing.List[BillingNoteBase]] = OMIT,
        place_of_service_code: FacilityTypeCode,
        patient_histories: typing.Optional[typing.List[PatientHistoryCategory]] = OMIT,
        service_lines: typing.Optional[typing.List[ServiceLineCreate]] = OMIT,
        guarantor: typing.Optional[GuarantorCreate] = OMIT,
        external_claim_submission: typing.Optional[ExternalClaimSubmissionCreate] = OMIT,
        external_id: EncounterExternalId,
        date_of_service: Date,
        end_date_of_service: typing.Optional[Date] = OMIT,
        prior_authorization_number: typing.Optional[PriorAuthorizationNumber] = OMIT,
        patient_authorized_release: bool,
        benefits_assigned_to_provider: bool,
        provider_accepts_assignment: bool,
        appointment_type: typing.Optional[str] = OMIT,
        existing_medications: typing.Optional[typing.List[Medication]] = OMIT,
        vitals: typing.Optional[Vitals] = OMIT,
        interventions: typing.Optional[typing.List[Intervention]] = OMIT,
        pay_to_address: typing.Optional[StreetAddressLongZip] = OMIT,
        synchronicity: typing.Optional[SynchronicityType] = OMIT,
        billable_status: BillableStatusType,
        responsible_party: ResponsiblePartyType,
        additional_information: typing.Optional[str] = OMIT,
        service_authorization_exception_code: typing.Optional[ServiceAuthorizationExceptionCode] = OMIT,
        admission_date: typing.Optional[Date] = OMIT,
        discharge_date: typing.Optional[Date] = OMIT,
        onset_of_current_illness_or_symptom_date: typing.Optional[Date] = OMIT,
        last_menstrual_period_date: typing.Optional[Date] = OMIT,
        delay_reason_code: typing.Optional[DelayReasonCode] = OMIT,
    ) -> Encounter:
        """
        Parameters:
            - patient: PatientCreate. Contains the identification information of the individual receiving medical services.

            - billing_provider: BillingProvider. The billing provider is the provider or business entity submitting the claim. Billing provider may be, but is not necessarily, the same person/NPI as the rendering provider. From a payer's perspective, this represents the person or entity being reimbursed. When a contract exists with the target payer, the billing provider should be the entity contracted with the payer. In some circumstances, this will be an individual provider. In that case, submit that provider's NPI and the tax ID (TIN) that the provider gave to the payer during contracting. In other cases, the billing entity will be a medical group. If so, submit the group NPI and the group's tax ID. Box 33 on the CMS-1500 claim form.

            - rendering_provider: RenderingProvider. The rendering provider is the practitioner -- physician, nurse practitioner, etc. -- performing the service.
                                                     For telehealth services, the rendering provider performs the visit, asynchronous communication, or other service. The rendering provider address should generally be the same as the service facility address.

            - referring_provider: typing.Optional[ReferringProvider]. The provider who referred the services that were rendered.
                                                                      All physicians who order services or refer Medicare beneficiaries must
                                                                      report this data.
                                                                      If a claim involves multiple referring physicians, create a separate
                                                                      encounter for each physician.

            - service_facility: typing.Optional[EncounterServiceFacilityBase]. Encounter Service facility is typically the location a medical service was rendered, such as a provider office or hospital. For telehealth, service facility can represent the provider's location when the service was delivered (e.g., home), or the location where an in-person visit would have taken place, whichever is easier to identify. If the provider is in-network, service facility may be defined in payer contracts. Box 32 on the CMS-1500 claim form. Note that for an in-network claim to be successfully adjudicated, the service facility address listed on claims must match what was provided to the payer during the credentialing process.

            - subscriber_primary: typing.Optional[SubscriberCreate]. Subscriber_primary is required when responsible_party is INSURANCE_PAY (i.e. when the claim should be billed to insurance).
                                                                     These are not required fields when responsible_party is SELF_PAY (i.e. when the claim should be billed to the patient).
                                                                     However, if you collect this for patients, even self-pay, we recommend including it when sending encounters to Candid.
                                                                     Note: Cash Pay is no longer a valid payer_id in v4, please use responsible party to define self-pay claims.

            - subscriber_secondary: typing.Optional[SubscriberCreate]. Please always include this when you have it, even for self-pay claims.

            - diagnoses: typing.List[DiagnosisCreate]. Ideally, this field should contain no more than 12 diagnoses. However, more diagnoses
                                                       may be submitted at this time, and coders will later prioritize the 12 that will be
                                                       submitted to the payor.

            - clinical_notes: typing.Optional[typing.List[ClinicalNoteCategoryCreate]]. Holds a collection of clinical observations made by healthcare providers during patient encounters.

            - billing_notes: typing.Optional[typing.List[BillingNoteBase]]. Spot to store misc, human-readable, notes about this encounter to be used
                                                                            in the billing process.

            - place_of_service_code: FacilityTypeCode. Box 24B on the CMS-1500 claim form. Line-level place of service is not currently supported. 02 for telemedicine, 11 for in-person. Full list [here](https://www.cms.gov/Medicare/Coding/place-of-service-codes/Place_of_Service_Code_Set).

            - patient_histories: typing.Optional[typing.List[PatientHistoryCategory]].

            - service_lines: typing.Optional[typing.List[ServiceLineCreate]]. Each service line must be linked to a diagnosis. Concretely,
                                                                              `service_line.diagnosis_pointers`must contain at least one entry which should be
                                                                              in bounds of the diagnoses list field.

            - guarantor: typing.Optional[GuarantorCreate]. Personal and contact info for the guarantor of the patient responsibility.

            - external_claim_submission: typing.Optional[ExternalClaimSubmissionCreate]. ***This field is in beta.***
                                                                                         To be included for claims that have been submitted outside of Candid.
                                                                                         Candid supports posting remits and payments to these claims and working them in-platform (e.g. editing, resubmitting).

            - external_id: EncounterExternalId. A client-specified unique ID to associate with this encounter;
                                                for example, your internal encounter ID or a Dr. Chrono encounter ID.
                                                This field should not contain PHI.
            - date_of_service: Date. Date formatted as YYYY-MM-DD; eg: 2019-08-24.
                                     This date must be the local date in the timezone where the service occurred.
                                     Box 24a on the CMS-1500 claim form.
                                     If service occurred over a range of dates, this should be the start date.
            - end_date_of_service: typing.Optional[Date]. Date formatted as YYYY-MM-DD; eg: 2019-08-25.
                                                          This date must be the local date in the timezone where the service occurred.
                                                          If omitted, the Encounter is assumed to be for a single day.
                                                          Must not be temporally before the date_of_service field.
            - prior_authorization_number: typing.Optional[PriorAuthorizationNumber]. Box 23 on the CMS-1500 claim form.

            - patient_authorized_release: bool. Whether this patient has authorized the release of medical information
                                                for billing purpose.
                                                Box 12 on the CMS-1500 claim form.
            - benefits_assigned_to_provider: bool. Whether this patient has authorized insurance payments to be made to you,
                                                   not them. If false, patient may receive reimbursement.
                                                   Box 13 on the CMS-1500 claim form.
            - provider_accepts_assignment: bool. Whether you have accepted the patient's authorization for insurance payments
                                                 to be made to you, not them.
                                                 Box 27 on the CMS-1500 claim form.
            - appointment_type: typing.Optional[str]. Human-readable description of the appointment type (ex: "Acupuncture - Headaches").

            - existing_medications: typing.Optional[typing.List[Medication]].

            - vitals: typing.Optional[Vitals].

            - interventions: typing.Optional[typing.List[Intervention]].

            - pay_to_address: typing.Optional[StreetAddressLongZip]. Specifies the address to which payments for the claim should be sent.

            - synchronicity: typing.Optional[SynchronicityType]. Whether or not this was a synchronous or asynchronous encounter.
                                                                 Asynchronous encounters occur when providers and patients communicate online using
                                                                 forms, instant messaging, or other pre-recorded digital mediums.
                                                                 Synchronous encounters occur in live, real-time settings where the patient interacts
                                                                 directly with the provider, such as over video or a phone call.
            - billable_status: BillableStatusType. Defines if the Encounter is to be billed by Candid to the responsible_party.
                                                   Examples for when this should be set to NOT_BILLABLE include
                                                   if the Encounter has not occurred yet or if there is no intention of ever billing the responsible_party.
            - responsible_party: ResponsiblePartyType. Defines the party to be billed with the initial balance owed on the claim.

            - additional_information: typing.Optional[str]. Defines additional information on the claim needed by the payer.
                                                            Box 19 on the CMS-1500 claim form.
            - service_authorization_exception_code: typing.Optional[ServiceAuthorizationExceptionCode]. 837p Loop2300 REF\*4N
                                                                                                        Required when mandated by government law or regulation to obtain authorization for specific service(s) but, for the
                                                                                                        reasons listed in one of the enum values of ServiceAuthorizationExceptionCode, the service was performed without
                                                                                                        obtaining the authorization.
            - admission_date: typing.Optional[Date]. 837p Loop2300 DTP\*435, CMS-1500 Box 18
                                                     Required on all ambulance claims when the patient was known to be admitted to the hospital.
                                                     OR
                                                     Required on all claims involving inpatient medical visits.
            - discharge_date: typing.Optional[Date]. 837p Loop2300 DTP\*096, CMS-1500 Box 18
                                                     Required for inpatient claims when the patient was discharged from the facility and the discharge date is known.
            - onset_of_current_illness_or_symptom_date: typing.Optional[Date]. 837p Loop2300 DTP\*431, CMS-1500 Box 14
                                                                               Required for the initial medical service or visit performed in response to a medical emergency when the date is available and is different than the date of service.
                                                                               OR
                                                                               This date is the onset of acute symptoms for the current illness or condition.
            - last_menstrual_period_date: typing.Optional[Date]. 837p Loop2300 DTP\*484, CMS-1500 Box 14
                                                                 Required when, in the judgment of the provider, the services on this claim are related to the patient's pregnancy.
            - delay_reason_code: typing.Optional[DelayReasonCode]. 837i Loop2300, CLM-1300 Box 20
                                                                   Code indicating the reason why a request was delayed
        """
        _request: typing.Dict[str, typing.Any] = {
            "patient": patient,
            "billing_provider": billing_provider,
            "rendering_provider": rendering_provider,
            "diagnoses": diagnoses,
            "place_of_service_code": place_of_service_code,
            "external_id": external_id,
            "date_of_service": date_of_service,
            "patient_authorized_release": patient_authorized_release,
            "benefits_assigned_to_provider": benefits_assigned_to_provider,
            "provider_accepts_assignment": provider_accepts_assignment,
            "billable_status": billable_status,
            "responsible_party": responsible_party,
        }
        if referring_provider is not OMIT:
            _request["referring_provider"] = referring_provider
        if service_facility is not OMIT:
            _request["service_facility"] = service_facility
        if subscriber_primary is not OMIT:
            _request["subscriber_primary"] = subscriber_primary
        if subscriber_secondary is not OMIT:
            _request["subscriber_secondary"] = subscriber_secondary
        if clinical_notes is not OMIT:
            _request["clinical_notes"] = clinical_notes
        if billing_notes is not OMIT:
            _request["billing_notes"] = billing_notes
        if patient_histories is not OMIT:
            _request["patient_histories"] = patient_histories
        if service_lines is not OMIT:
            _request["service_lines"] = service_lines
        if guarantor is not OMIT:
            _request["guarantor"] = guarantor
        if external_claim_submission is not OMIT:
            _request["external_claim_submission"] = external_claim_submission
        if end_date_of_service is not OMIT:
            _request["end_date_of_service"] = end_date_of_service
        if prior_authorization_number is not OMIT:
            _request["prior_authorization_number"] = prior_authorization_number
        if appointment_type is not OMIT:
            _request["appointment_type"] = appointment_type
        if existing_medications is not OMIT:
            _request["existing_medications"] = existing_medications
        if vitals is not OMIT:
            _request["vitals"] = vitals
        if interventions is not OMIT:
            _request["interventions"] = interventions
        if pay_to_address is not OMIT:
            _request["pay_to_address"] = pay_to_address
        if synchronicity is not OMIT:
            _request["synchronicity"] = synchronicity
        if additional_information is not OMIT:
            _request["additional_information"] = additional_information
        if service_authorization_exception_code is not OMIT:
            _request["service_authorization_exception_code"] = service_authorization_exception_code
        if admission_date is not OMIT:
            _request["admission_date"] = admission_date
        if discharge_date is not OMIT:
            _request["discharge_date"] = discharge_date
        if onset_of_current_illness_or_symptom_date is not OMIT:
            _request["onset_of_current_illness_or_symptom_date"] = onset_of_current_illness_or_symptom_date
        if last_menstrual_period_date is not OMIT:
            _request["last_menstrual_period_date"] = last_menstrual_period_date
        if delay_reason_code is not OMIT:
            _request["delay_reason_code"] = delay_reason_code
        _response = self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "api/encounters/v4"),
            json=jsonable_encoder(_request),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(Encounter, _response_json)  # type: ignore
        if "errorName" in _response_json:
            if _response_json["errorName"] == "EncounterExternalIdUniquenessError":
                raise EncounterExternalIdUniquenessError(
                    pydantic.parse_obj_as(EncounterExternalIdUniquenessErrorType, _response_json["content"])  # type: ignore
                )
            if _response_json["errorName"] == "EncounterPatientControlNumberUniquenessError":
                raise EncounterPatientControlNumberUniquenessError(
                    pydantic.parse_obj_as(EncounterPatientControlNumberUniquenessErrorType, _response_json["content"])  # type: ignore
                )
            if _response_json["errorName"] == "EntityNotFoundError":
                raise EntityNotFoundError(
                    pydantic.parse_obj_as(EntityNotFoundErrorMessage, _response_json["content"])  # type: ignore
                )
            if _response_json["errorName"] == "EncounterGuarantorMissingContactInfoError":
                raise EncounterGuarantorMissingContactInfoError(
                    pydantic.parse_obj_as(EncounterGuarantorMissingContactInfoErrorType, _response_json["content"])  # type: ignore
                )
            if _response_json["errorName"] == "HttpRequestValidationsError":
                raise HttpRequestValidationsError(
                    pydantic.parse_obj_as(typing.List[RequestValidationError], _response_json["content"])  # type: ignore
                )
            if _response_json["errorName"] == "CashPayPayerError":
                raise CashPayPayerError(
                    pydantic.parse_obj_as(CashPayPayerErrorMessage, _response_json["content"])  # type: ignore
                )
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def update(
        self,
        encounter_id: EncounterId,
        *,
        prior_authorization_number: typing.Optional[PriorAuthorizationNumber] = OMIT,
        external_id: typing.Optional[EncounterExternalId] = OMIT,
        date_of_service: typing.Optional[Date] = OMIT,
        diagnosis_ids: typing.Optional[typing.List[DiagnosisId]] = OMIT,
        tag_ids: typing.Optional[typing.List[TagId]] = OMIT,
        clinical_notes: typing.Optional[typing.List[ClinicalNoteCategoryCreate]] = OMIT,
        pay_to_address: typing.Optional[StreetAddressLongZip] = OMIT,
        billable_status: typing.Optional[BillableStatusType] = OMIT,
        responsible_party: typing.Optional[ResponsiblePartyType] = OMIT,
        provider_accepts_assignment: typing.Optional[bool] = OMIT,
        benefits_assigned_to_provider: typing.Optional[bool] = OMIT,
        synchronicity: typing.Optional[SynchronicityType] = OMIT,
        place_of_service_code: typing.Optional[FacilityTypeCode] = OMIT,
        appointment_type: typing.Optional[str] = OMIT,
        end_date_of_service: typing.Optional[Date] = OMIT,
        subscriber_primary: typing.Optional[SubscriberCreate] = OMIT,
        subscriber_secondary: typing.Optional[SubscriberCreate] = OMIT,
        additional_information: typing.Optional[str] = OMIT,
        service_authorization_exception_code: typing.Optional[ServiceAuthorizationExceptionCode] = OMIT,
        admission_date: typing.Optional[Date] = OMIT,
        discharge_date: typing.Optional[Date] = OMIT,
        onset_of_current_illness_or_symptom_date: typing.Optional[Date] = OMIT,
        last_menstrual_period_date: typing.Optional[Date] = OMIT,
        delay_reason_code: typing.Optional[DelayReasonCode] = OMIT,
    ) -> Encounter:
        """
        Parameters:
            - encounter_id: EncounterId.

            - prior_authorization_number: typing.Optional[PriorAuthorizationNumber]. Box 23 on the CMS-1500 claim form.

            - external_id: typing.Optional[EncounterExternalId]. A client-specified unique ID to associate with this encounter;
                                                                 for example, your internal encounter ID or a Dr. Chrono encounter ID.
                                                                 This field should not contain PHI.

            - date_of_service: typing.Optional[Date]. Date formatted as YYYY-MM-DD; eg: 2019-08-24.
                                                      This date must be the local date in the timezone where the service occurred.
                                                      Box 24a on the CMS-1500 claim form.
                                                      If service occurred over a range of dates, this should be the start date.

            - diagnosis_ids: typing.Optional[typing.List[DiagnosisId]]. Ideally, this field should contain no more than 12 diagnoses. However, more diagnoses
                                                                        may be submitted at this time, and coders will later prioritize the 12 that will be
                                                                        submitted to the payor.

            - tag_ids: typing.Optional[typing.List[TagId]]. Names of tags that should be on the encounter.

            - clinical_notes: typing.Optional[typing.List[ClinicalNoteCategoryCreate]]. Holds a collection of clinical observations made by healthcare providers during patient encounters.

            - pay_to_address: typing.Optional[StreetAddressLongZip]. Specifies the address to which payments for the claim should be sent.

            - billable_status: typing.Optional[BillableStatusType]. Defines if the Encounter is to be billed by Candid to the responsible_party. Examples for when this should be set to NOT_BILLABLE include if the Encounter has not occurred yet or if there is no intention of ever billing the responsible_party.

            - responsible_party: typing.Optional[ResponsiblePartyType]. Defines the party to be billed with the initial balance owed on the claim. Use SELF_PAY if you intend to bill self pay/cash pay.

            - provider_accepts_assignment: typing.Optional[bool]. Whether you have accepted the patient's authorization for insurance payments to be made to you, not them. Box 27 on the CMS-1500 claim form.

            - benefits_assigned_to_provider: typing.Optional[bool]. Whether this patient has authorized insurance payments to be made to you, not them. If false, patient may receive reimbursement. Box 13 on the CMS-1500 claim form.

            - synchronicity: typing.Optional[SynchronicityType]. Whether or not this was a synchronous or asynchronous encounter. Asynchronous encounters occur when providers and patients communicate online using forms, instant messaging, or other pre-recorded digital mediums. Synchronous encounters occur in live, real-time settings where the patient interacts directly with the provider, such as over video or a phone call.

            - place_of_service_code: typing.Optional[FacilityTypeCode]. Box 24B on the CMS-1500 claim form. Line-level place of service is not currently supported. 02 for telemedicine, 11 for in-person. Full list [here](https://www.cms.gov/Medicare/Coding/place-of-service-codes/Place_of_Service_Code_Set).

            - appointment_type: typing.Optional[str]. Human-readable description of the appointment type (ex: "Acupuncture - Headaches").

            - end_date_of_service: typing.Optional[Date]. Date formatted as YYYY-MM-DD; eg: 2019-08-25.
                                                          This date must be the local date in the timezone where the service occurred.
                                                          If omitted, the Encounter is assumed to be for a single day.
                                                          Must not be temporally before the date_of_service field.

            - subscriber_primary: typing.Optional[SubscriberCreate]. Contains details of the primary insurance subscriber.

            - subscriber_secondary: typing.Optional[SubscriberCreate]. Contains details of the secondary insurance subscriber.

            - additional_information: typing.Optional[str]. Defines additional information on the claim needed by the payer.
                                                            Box 19 on the CMS-1500 claim form.

            - service_authorization_exception_code: typing.Optional[ServiceAuthorizationExceptionCode]. 837p Loop2300 REF*4N
                                                                                                        Required when mandated by government law or regulation to obtain authorization for specific service(s) but, for the
                                                                                                        reasons listed in one of the enum values of ServiceAuthorizationExceptionCode, the service was performed without
                                                                                                        obtaining the authorization.

            - admission_date: typing.Optional[Date]. 837p Loop2300 DTP*435, CMS-1500 Box 18
                                                     Required on all ambulance claims when the patient was known to be admitted to the hospital.
                                                     OR
                                                     Required on all claims involving inpatient medical visits.

            - discharge_date: typing.Optional[Date]. 837p Loop2300 DTP*096, CMS-1500 Box 18
                                                     Required for inpatient claims when the patient was discharged from the facility and the discharge date is known.

            - onset_of_current_illness_or_symptom_date: typing.Optional[Date]. 837p Loop2300 DTP*431, CMS-1500 Box 14
                                                                               Required for the initial medical service or visit performed in response to a medical emergency when the date is available and is different than the date of service.
                                                                               OR
                                                                               This date is the onset of acute symptoms for the current illness or condition.

            - last_menstrual_period_date: typing.Optional[Date]. 837p Loop2300 DTP*484, CMS-1500 Box 14
                                                                 Required when, in the judgment of the provider, the services on this claim are related to the patient's pregnancy.

            - delay_reason_code: typing.Optional[DelayReasonCode]. 837i Loop2300, CLM-1300 Box 20
                                                                   Code indicating the reason why a request was delayed

        """
        _request: typing.Dict[str, typing.Any] = {}
        if prior_authorization_number is not OMIT:
            _request["prior_authorization_number"] = prior_authorization_number
        if external_id is not OMIT:
            _request["external_id"] = external_id
        if date_of_service is not OMIT:
            _request["date_of_service"] = date_of_service
        if diagnosis_ids is not OMIT:
            _request["diagnosis_ids"] = diagnosis_ids
        if tag_ids is not OMIT:
            _request["tag_ids"] = tag_ids
        if clinical_notes is not OMIT:
            _request["clinical_notes"] = clinical_notes
        if pay_to_address is not OMIT:
            _request["pay_to_address"] = pay_to_address
        if billable_status is not OMIT:
            _request["billable_status"] = billable_status
        if responsible_party is not OMIT:
            _request["responsible_party"] = responsible_party
        if provider_accepts_assignment is not OMIT:
            _request["provider_accepts_assignment"] = provider_accepts_assignment
        if benefits_assigned_to_provider is not OMIT:
            _request["benefits_assigned_to_provider"] = benefits_assigned_to_provider
        if synchronicity is not OMIT:
            _request["synchronicity"] = synchronicity
        if place_of_service_code is not OMIT:
            _request["place_of_service_code"] = place_of_service_code
        if appointment_type is not OMIT:
            _request["appointment_type"] = appointment_type
        if end_date_of_service is not OMIT:
            _request["end_date_of_service"] = end_date_of_service
        if subscriber_primary is not OMIT:
            _request["subscriber_primary"] = subscriber_primary
        if subscriber_secondary is not OMIT:
            _request["subscriber_secondary"] = subscriber_secondary
        if additional_information is not OMIT:
            _request["additional_information"] = additional_information
        if service_authorization_exception_code is not OMIT:
            _request["service_authorization_exception_code"] = service_authorization_exception_code
        if admission_date is not OMIT:
            _request["admission_date"] = admission_date
        if discharge_date is not OMIT:
            _request["discharge_date"] = discharge_date
        if onset_of_current_illness_or_symptom_date is not OMIT:
            _request["onset_of_current_illness_or_symptom_date"] = onset_of_current_illness_or_symptom_date
        if last_menstrual_period_date is not OMIT:
            _request["last_menstrual_period_date"] = last_menstrual_period_date
        if delay_reason_code is not OMIT:
            _request["delay_reason_code"] = delay_reason_code
        _response = self._client_wrapper.httpx_client.request(
            "PATCH",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", f"api/encounters/v4/{encounter_id}"),
            json=jsonable_encoder(_request),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(Encounter, _response_json)  # type: ignore
        if "errorName" in _response_json:
            if _response_json["errorName"] == "EncounterExternalIdUniquenessError":
                raise EncounterExternalIdUniquenessError(
                    pydantic.parse_obj_as(EncounterExternalIdUniquenessErrorType, _response_json["content"])  # type: ignore
                )
            if _response_json["errorName"] == "EntityNotFoundError":
                raise EntityNotFoundError(
                    pydantic.parse_obj_as(EntityNotFoundErrorMessage, _response_json["content"])  # type: ignore
                )
            if _response_json["errorName"] == "UnauthorizedError":
                raise UnauthorizedError(
                    pydantic.parse_obj_as(UnauthorizedErrorMessage, _response_json["content"])  # type: ignore
                )
            if _response_json["errorName"] == "HttpRequestValidationsError":
                raise HttpRequestValidationsError(
                    pydantic.parse_obj_as(typing.List[RequestValidationError], _response_json["content"])  # type: ignore
                )
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncV4Client:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def get_all(
        self,
        *,
        limit: typing.Optional[int] = None,
        claim_status: typing.Optional[ClaimStatus] = None,
        sort: typing.Optional[EncounterSortOptions] = None,
        page_token: typing.Optional[PageToken] = None,
        date_of_service_min: typing.Optional[Date] = None,
        date_of_service_max: typing.Optional[Date] = None,
        primary_payer_names: typing.Optional[str] = None,
        search_term: typing.Optional[str] = None,
        external_id: typing.Optional[EncounterExternalId] = None,
        diagnoses_updated_since: typing.Optional[dt.datetime] = None,
        tag_ids: typing.Optional[typing.Union[TagId, typing.List[TagId]]] = None,
        work_queue_id: typing.Optional[WorkQueueId] = None,
        billable_status: typing.Optional[BillableStatusType] = None,
        responsible_party: typing.Optional[ResponsiblePartyType] = None,
        owner_of_next_action: typing.Optional[EncounterOwnerOfNextActionType] = None,
    ) -> EncounterPage:
        """
        Parameters:
            - limit: typing.Optional[int]. Maximum number of entities per page, defaults to 100.

            - claim_status: typing.Optional[ClaimStatus]. Indicates the current status of an insurance claim within the billing process.

            - sort: typing.Optional[EncounterSortOptions]. Defaults to created_at:desc.

            - page_token: typing.Optional[PageToken].

            - date_of_service_min: typing.Optional[Date]. Date formatted as YYYY-MM-DD; eg: 2019-08-25.

            - date_of_service_max: typing.Optional[Date]. Date formatted as YYYY-MM-DD; eg: 2019-08-25.

            - primary_payer_names: typing.Optional[str]. Comma delimited string.

            - search_term: typing.Optional[str]. Filter by any of the following fields: encounter_id, claim_id, patient external_id,
                                                 patient date of birth, patient first name, patient last name,
                                                 or encounter external id.
            - external_id: typing.Optional[EncounterExternalId]. Filter to an exact match on encounter external_id, if one exists.

            - diagnoses_updated_since: typing.Optional[dt.datetime]. ISO 8601 timestamp; ideally in UTC (although not required): 2019-08-24T14:15:22Z.

            - tag_ids: typing.Optional[typing.Union[TagId, typing.List[TagId]]]. Filter by name of tags on encounters.

            - work_queue_id: typing.Optional[WorkQueueId].

            - billable_status: typing.Optional[BillableStatusType]. Defines if the Encounter is to be billed by Candid to the responsible_party. Examples for when this should be set to NOT_BILLABLE include if the Encounter has not occurred yet or if there is no intention of ever billing the responsible_party.

            - responsible_party: typing.Optional[ResponsiblePartyType]. Defines the party to be billed with the initial balance owed on the claim. Use SELF_PAY if you intend to bill self pay/cash pay.

            - owner_of_next_action: typing.Optional[EncounterOwnerOfNextActionType]. The party who is responsible for taking the next action on an Encounter, as defined by ownership of open Tasks.
        ---
        import datetime

        from candid import ClaimStatus
        from candid.client import AsyncCandidApi
        from candid.resources.encounters.v_4 import EncounterSortOptions

        client = AsyncCandidApi(
            token="YOUR_TOKEN",
        )
        await client.encounters.v_4.get_all(
            limit=100,
            claim_status=ClaimStatus.BILLER_RECEIVED,
            sort=EncounterSortOptions.CREATED_AT_ASC,
            page_token="eyJ0b2tlbiI6IjEiLCJwYWdlX3Rva2VuIjoiMiJ9",
            date_of_service_min="2019-08-24",
            date_of_service_max="2019-08-25",
            primary_payer_names="Medicare,Medicaid",
            search_term="doe",
            external_id="123456",
            diagnoses_updated_since=datetime.datetime.fromisoformat(
                "2019-08-24 14:15:22+00:00",
            ),
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "api/encounters/v4"),
            params=remove_none_from_dict(
                {
                    "limit": limit,
                    "claim_status": claim_status,
                    "sort": sort,
                    "page_token": page_token,
                    "date_of_service_min": date_of_service_min,
                    "date_of_service_max": date_of_service_max,
                    "primary_payer_names": primary_payer_names,
                    "search_term": search_term,
                    "external_id": external_id,
                    "diagnoses_updated_since": serialize_datetime(diagnoses_updated_since)
                    if diagnoses_updated_since is not None
                    else None,
                    "tag_ids": tag_ids,
                    "work_queue_id": work_queue_id,
                    "billable_status": billable_status,
                    "responsible_party": responsible_party,
                    "owner_of_next_action": owner_of_next_action,
                }
            ),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(EncounterPage, _response_json)  # type: ignore
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get(self, encounter_id: EncounterId) -> Encounter:
        """
        Parameters:
            - encounter_id: EncounterId.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", f"api/encounters/v4/{encounter_id}"),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(Encounter, _response_json)  # type: ignore
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def create(
        self,
        *,
        patient: PatientCreate,
        billing_provider: BillingProvider,
        rendering_provider: RenderingProvider,
        referring_provider: typing.Optional[ReferringProvider] = OMIT,
        service_facility: typing.Optional[EncounterServiceFacilityBase] = OMIT,
        subscriber_primary: typing.Optional[SubscriberCreate] = OMIT,
        subscriber_secondary: typing.Optional[SubscriberCreate] = OMIT,
        diagnoses: typing.List[DiagnosisCreate],
        clinical_notes: typing.Optional[typing.List[ClinicalNoteCategoryCreate]] = OMIT,
        billing_notes: typing.Optional[typing.List[BillingNoteBase]] = OMIT,
        place_of_service_code: FacilityTypeCode,
        patient_histories: typing.Optional[typing.List[PatientHistoryCategory]] = OMIT,
        service_lines: typing.Optional[typing.List[ServiceLineCreate]] = OMIT,
        guarantor: typing.Optional[GuarantorCreate] = OMIT,
        external_claim_submission: typing.Optional[ExternalClaimSubmissionCreate] = OMIT,
        external_id: EncounterExternalId,
        date_of_service: Date,
        end_date_of_service: typing.Optional[Date] = OMIT,
        prior_authorization_number: typing.Optional[PriorAuthorizationNumber] = OMIT,
        patient_authorized_release: bool,
        benefits_assigned_to_provider: bool,
        provider_accepts_assignment: bool,
        appointment_type: typing.Optional[str] = OMIT,
        existing_medications: typing.Optional[typing.List[Medication]] = OMIT,
        vitals: typing.Optional[Vitals] = OMIT,
        interventions: typing.Optional[typing.List[Intervention]] = OMIT,
        pay_to_address: typing.Optional[StreetAddressLongZip] = OMIT,
        synchronicity: typing.Optional[SynchronicityType] = OMIT,
        billable_status: BillableStatusType,
        responsible_party: ResponsiblePartyType,
        additional_information: typing.Optional[str] = OMIT,
        service_authorization_exception_code: typing.Optional[ServiceAuthorizationExceptionCode] = OMIT,
        admission_date: typing.Optional[Date] = OMIT,
        discharge_date: typing.Optional[Date] = OMIT,
        onset_of_current_illness_or_symptom_date: typing.Optional[Date] = OMIT,
        last_menstrual_period_date: typing.Optional[Date] = OMIT,
        delay_reason_code: typing.Optional[DelayReasonCode] = OMIT,
    ) -> Encounter:
        """
        Parameters:
            - patient: PatientCreate. Contains the identification information of the individual receiving medical services.

            - billing_provider: BillingProvider. The billing provider is the provider or business entity submitting the claim. Billing provider may be, but is not necessarily, the same person/NPI as the rendering provider. From a payer's perspective, this represents the person or entity being reimbursed. When a contract exists with the target payer, the billing provider should be the entity contracted with the payer. In some circumstances, this will be an individual provider. In that case, submit that provider's NPI and the tax ID (TIN) that the provider gave to the payer during contracting. In other cases, the billing entity will be a medical group. If so, submit the group NPI and the group's tax ID. Box 33 on the CMS-1500 claim form.

            - rendering_provider: RenderingProvider. The rendering provider is the practitioner -- physician, nurse practitioner, etc. -- performing the service.
                                                     For telehealth services, the rendering provider performs the visit, asynchronous communication, or other service. The rendering provider address should generally be the same as the service facility address.

            - referring_provider: typing.Optional[ReferringProvider]. The provider who referred the services that were rendered.
                                                                      All physicians who order services or refer Medicare beneficiaries must
                                                                      report this data.
                                                                      If a claim involves multiple referring physicians, create a separate
                                                                      encounter for each physician.

            - service_facility: typing.Optional[EncounterServiceFacilityBase]. Encounter Service facility is typically the location a medical service was rendered, such as a provider office or hospital. For telehealth, service facility can represent the provider's location when the service was delivered (e.g., home), or the location where an in-person visit would have taken place, whichever is easier to identify. If the provider is in-network, service facility may be defined in payer contracts. Box 32 on the CMS-1500 claim form. Note that for an in-network claim to be successfully adjudicated, the service facility address listed on claims must match what was provided to the payer during the credentialing process.

            - subscriber_primary: typing.Optional[SubscriberCreate]. Subscriber_primary is required when responsible_party is INSURANCE_PAY (i.e. when the claim should be billed to insurance).
                                                                     These are not required fields when responsible_party is SELF_PAY (i.e. when the claim should be billed to the patient).
                                                                     However, if you collect this for patients, even self-pay, we recommend including it when sending encounters to Candid.
                                                                     Note: Cash Pay is no longer a valid payer_id in v4, please use responsible party to define self-pay claims.

            - subscriber_secondary: typing.Optional[SubscriberCreate]. Please always include this when you have it, even for self-pay claims.

            - diagnoses: typing.List[DiagnosisCreate]. Ideally, this field should contain no more than 12 diagnoses. However, more diagnoses
                                                       may be submitted at this time, and coders will later prioritize the 12 that will be
                                                       submitted to the payor.

            - clinical_notes: typing.Optional[typing.List[ClinicalNoteCategoryCreate]]. Holds a collection of clinical observations made by healthcare providers during patient encounters.

            - billing_notes: typing.Optional[typing.List[BillingNoteBase]]. Spot to store misc, human-readable, notes about this encounter to be used
                                                                            in the billing process.

            - place_of_service_code: FacilityTypeCode. Box 24B on the CMS-1500 claim form. Line-level place of service is not currently supported. 02 for telemedicine, 11 for in-person. Full list [here](https://www.cms.gov/Medicare/Coding/place-of-service-codes/Place_of_Service_Code_Set).

            - patient_histories: typing.Optional[typing.List[PatientHistoryCategory]].

            - service_lines: typing.Optional[typing.List[ServiceLineCreate]]. Each service line must be linked to a diagnosis. Concretely,
                                                                              `service_line.diagnosis_pointers`must contain at least one entry which should be
                                                                              in bounds of the diagnoses list field.

            - guarantor: typing.Optional[GuarantorCreate]. Personal and contact info for the guarantor of the patient responsibility.

            - external_claim_submission: typing.Optional[ExternalClaimSubmissionCreate]. ***This field is in beta.***
                                                                                         To be included for claims that have been submitted outside of Candid.
                                                                                         Candid supports posting remits and payments to these claims and working them in-platform (e.g. editing, resubmitting).

            - external_id: EncounterExternalId. A client-specified unique ID to associate with this encounter;
                                                for example, your internal encounter ID or a Dr. Chrono encounter ID.
                                                This field should not contain PHI.
            - date_of_service: Date. Date formatted as YYYY-MM-DD; eg: 2019-08-24.
                                     This date must be the local date in the timezone where the service occurred.
                                     Box 24a on the CMS-1500 claim form.
                                     If service occurred over a range of dates, this should be the start date.
            - end_date_of_service: typing.Optional[Date]. Date formatted as YYYY-MM-DD; eg: 2019-08-25.
                                                          This date must be the local date in the timezone where the service occurred.
                                                          If omitted, the Encounter is assumed to be for a single day.
                                                          Must not be temporally before the date_of_service field.
            - prior_authorization_number: typing.Optional[PriorAuthorizationNumber]. Box 23 on the CMS-1500 claim form.

            - patient_authorized_release: bool. Whether this patient has authorized the release of medical information
                                                for billing purpose.
                                                Box 12 on the CMS-1500 claim form.
            - benefits_assigned_to_provider: bool. Whether this patient has authorized insurance payments to be made to you,
                                                   not them. If false, patient may receive reimbursement.
                                                   Box 13 on the CMS-1500 claim form.
            - provider_accepts_assignment: bool. Whether you have accepted the patient's authorization for insurance payments
                                                 to be made to you, not them.
                                                 Box 27 on the CMS-1500 claim form.
            - appointment_type: typing.Optional[str]. Human-readable description of the appointment type (ex: "Acupuncture - Headaches").

            - existing_medications: typing.Optional[typing.List[Medication]].

            - vitals: typing.Optional[Vitals].

            - interventions: typing.Optional[typing.List[Intervention]].

            - pay_to_address: typing.Optional[StreetAddressLongZip]. Specifies the address to which payments for the claim should be sent.

            - synchronicity: typing.Optional[SynchronicityType]. Whether or not this was a synchronous or asynchronous encounter.
                                                                 Asynchronous encounters occur when providers and patients communicate online using
                                                                 forms, instant messaging, or other pre-recorded digital mediums.
                                                                 Synchronous encounters occur in live, real-time settings where the patient interacts
                                                                 directly with the provider, such as over video or a phone call.
            - billable_status: BillableStatusType. Defines if the Encounter is to be billed by Candid to the responsible_party.
                                                   Examples for when this should be set to NOT_BILLABLE include
                                                   if the Encounter has not occurred yet or if there is no intention of ever billing the responsible_party.
            - responsible_party: ResponsiblePartyType. Defines the party to be billed with the initial balance owed on the claim.

            - additional_information: typing.Optional[str]. Defines additional information on the claim needed by the payer.
                                                            Box 19 on the CMS-1500 claim form.
            - service_authorization_exception_code: typing.Optional[ServiceAuthorizationExceptionCode]. 837p Loop2300 REF\*4N
                                                                                                        Required when mandated by government law or regulation to obtain authorization for specific service(s) but, for the
                                                                                                        reasons listed in one of the enum values of ServiceAuthorizationExceptionCode, the service was performed without
                                                                                                        obtaining the authorization.
            - admission_date: typing.Optional[Date]. 837p Loop2300 DTP\*435, CMS-1500 Box 18
                                                     Required on all ambulance claims when the patient was known to be admitted to the hospital.
                                                     OR
                                                     Required on all claims involving inpatient medical visits.
            - discharge_date: typing.Optional[Date]. 837p Loop2300 DTP\*096, CMS-1500 Box 18
                                                     Required for inpatient claims when the patient was discharged from the facility and the discharge date is known.
            - onset_of_current_illness_or_symptom_date: typing.Optional[Date]. 837p Loop2300 DTP\*431, CMS-1500 Box 14
                                                                               Required for the initial medical service or visit performed in response to a medical emergency when the date is available and is different than the date of service.
                                                                               OR
                                                                               This date is the onset of acute symptoms for the current illness or condition.
            - last_menstrual_period_date: typing.Optional[Date]. 837p Loop2300 DTP\*484, CMS-1500 Box 14
                                                                 Required when, in the judgment of the provider, the services on this claim are related to the patient's pregnancy.
            - delay_reason_code: typing.Optional[DelayReasonCode]. 837i Loop2300, CLM-1300 Box 20
                                                                   Code indicating the reason why a request was delayed
        """
        _request: typing.Dict[str, typing.Any] = {
            "patient": patient,
            "billing_provider": billing_provider,
            "rendering_provider": rendering_provider,
            "diagnoses": diagnoses,
            "place_of_service_code": place_of_service_code,
            "external_id": external_id,
            "date_of_service": date_of_service,
            "patient_authorized_release": patient_authorized_release,
            "benefits_assigned_to_provider": benefits_assigned_to_provider,
            "provider_accepts_assignment": provider_accepts_assignment,
            "billable_status": billable_status,
            "responsible_party": responsible_party,
        }
        if referring_provider is not OMIT:
            _request["referring_provider"] = referring_provider
        if service_facility is not OMIT:
            _request["service_facility"] = service_facility
        if subscriber_primary is not OMIT:
            _request["subscriber_primary"] = subscriber_primary
        if subscriber_secondary is not OMIT:
            _request["subscriber_secondary"] = subscriber_secondary
        if clinical_notes is not OMIT:
            _request["clinical_notes"] = clinical_notes
        if billing_notes is not OMIT:
            _request["billing_notes"] = billing_notes
        if patient_histories is not OMIT:
            _request["patient_histories"] = patient_histories
        if service_lines is not OMIT:
            _request["service_lines"] = service_lines
        if guarantor is not OMIT:
            _request["guarantor"] = guarantor
        if external_claim_submission is not OMIT:
            _request["external_claim_submission"] = external_claim_submission
        if end_date_of_service is not OMIT:
            _request["end_date_of_service"] = end_date_of_service
        if prior_authorization_number is not OMIT:
            _request["prior_authorization_number"] = prior_authorization_number
        if appointment_type is not OMIT:
            _request["appointment_type"] = appointment_type
        if existing_medications is not OMIT:
            _request["existing_medications"] = existing_medications
        if vitals is not OMIT:
            _request["vitals"] = vitals
        if interventions is not OMIT:
            _request["interventions"] = interventions
        if pay_to_address is not OMIT:
            _request["pay_to_address"] = pay_to_address
        if synchronicity is not OMIT:
            _request["synchronicity"] = synchronicity
        if additional_information is not OMIT:
            _request["additional_information"] = additional_information
        if service_authorization_exception_code is not OMIT:
            _request["service_authorization_exception_code"] = service_authorization_exception_code
        if admission_date is not OMIT:
            _request["admission_date"] = admission_date
        if discharge_date is not OMIT:
            _request["discharge_date"] = discharge_date
        if onset_of_current_illness_or_symptom_date is not OMIT:
            _request["onset_of_current_illness_or_symptom_date"] = onset_of_current_illness_or_symptom_date
        if last_menstrual_period_date is not OMIT:
            _request["last_menstrual_period_date"] = last_menstrual_period_date
        if delay_reason_code is not OMIT:
            _request["delay_reason_code"] = delay_reason_code
        _response = await self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "api/encounters/v4"),
            json=jsonable_encoder(_request),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(Encounter, _response_json)  # type: ignore
        if "errorName" in _response_json:
            if _response_json["errorName"] == "EncounterExternalIdUniquenessError":
                raise EncounterExternalIdUniquenessError(
                    pydantic.parse_obj_as(EncounterExternalIdUniquenessErrorType, _response_json["content"])  # type: ignore
                )
            if _response_json["errorName"] == "EncounterPatientControlNumberUniquenessError":
                raise EncounterPatientControlNumberUniquenessError(
                    pydantic.parse_obj_as(EncounterPatientControlNumberUniquenessErrorType, _response_json["content"])  # type: ignore
                )
            if _response_json["errorName"] == "EntityNotFoundError":
                raise EntityNotFoundError(
                    pydantic.parse_obj_as(EntityNotFoundErrorMessage, _response_json["content"])  # type: ignore
                )
            if _response_json["errorName"] == "EncounterGuarantorMissingContactInfoError":
                raise EncounterGuarantorMissingContactInfoError(
                    pydantic.parse_obj_as(EncounterGuarantorMissingContactInfoErrorType, _response_json["content"])  # type: ignore
                )
            if _response_json["errorName"] == "HttpRequestValidationsError":
                raise HttpRequestValidationsError(
                    pydantic.parse_obj_as(typing.List[RequestValidationError], _response_json["content"])  # type: ignore
                )
            if _response_json["errorName"] == "CashPayPayerError":
                raise CashPayPayerError(
                    pydantic.parse_obj_as(CashPayPayerErrorMessage, _response_json["content"])  # type: ignore
                )
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def update(
        self,
        encounter_id: EncounterId,
        *,
        prior_authorization_number: typing.Optional[PriorAuthorizationNumber] = OMIT,
        external_id: typing.Optional[EncounterExternalId] = OMIT,
        date_of_service: typing.Optional[Date] = OMIT,
        diagnosis_ids: typing.Optional[typing.List[DiagnosisId]] = OMIT,
        tag_ids: typing.Optional[typing.List[TagId]] = OMIT,
        clinical_notes: typing.Optional[typing.List[ClinicalNoteCategoryCreate]] = OMIT,
        pay_to_address: typing.Optional[StreetAddressLongZip] = OMIT,
        billable_status: typing.Optional[BillableStatusType] = OMIT,
        responsible_party: typing.Optional[ResponsiblePartyType] = OMIT,
        provider_accepts_assignment: typing.Optional[bool] = OMIT,
        benefits_assigned_to_provider: typing.Optional[bool] = OMIT,
        synchronicity: typing.Optional[SynchronicityType] = OMIT,
        place_of_service_code: typing.Optional[FacilityTypeCode] = OMIT,
        appointment_type: typing.Optional[str] = OMIT,
        end_date_of_service: typing.Optional[Date] = OMIT,
        subscriber_primary: typing.Optional[SubscriberCreate] = OMIT,
        subscriber_secondary: typing.Optional[SubscriberCreate] = OMIT,
        additional_information: typing.Optional[str] = OMIT,
        service_authorization_exception_code: typing.Optional[ServiceAuthorizationExceptionCode] = OMIT,
        admission_date: typing.Optional[Date] = OMIT,
        discharge_date: typing.Optional[Date] = OMIT,
        onset_of_current_illness_or_symptom_date: typing.Optional[Date] = OMIT,
        last_menstrual_period_date: typing.Optional[Date] = OMIT,
        delay_reason_code: typing.Optional[DelayReasonCode] = OMIT,
    ) -> Encounter:
        """
        Parameters:
            - encounter_id: EncounterId.

            - prior_authorization_number: typing.Optional[PriorAuthorizationNumber]. Box 23 on the CMS-1500 claim form.

            - external_id: typing.Optional[EncounterExternalId]. A client-specified unique ID to associate with this encounter;
                                                                 for example, your internal encounter ID or a Dr. Chrono encounter ID.
                                                                 This field should not contain PHI.

            - date_of_service: typing.Optional[Date]. Date formatted as YYYY-MM-DD; eg: 2019-08-24.
                                                      This date must be the local date in the timezone where the service occurred.
                                                      Box 24a on the CMS-1500 claim form.
                                                      If service occurred over a range of dates, this should be the start date.

            - diagnosis_ids: typing.Optional[typing.List[DiagnosisId]]. Ideally, this field should contain no more than 12 diagnoses. However, more diagnoses
                                                                        may be submitted at this time, and coders will later prioritize the 12 that will be
                                                                        submitted to the payor.

            - tag_ids: typing.Optional[typing.List[TagId]]. Names of tags that should be on the encounter.

            - clinical_notes: typing.Optional[typing.List[ClinicalNoteCategoryCreate]]. Holds a collection of clinical observations made by healthcare providers during patient encounters.

            - pay_to_address: typing.Optional[StreetAddressLongZip]. Specifies the address to which payments for the claim should be sent.

            - billable_status: typing.Optional[BillableStatusType]. Defines if the Encounter is to be billed by Candid to the responsible_party. Examples for when this should be set to NOT_BILLABLE include if the Encounter has not occurred yet or if there is no intention of ever billing the responsible_party.

            - responsible_party: typing.Optional[ResponsiblePartyType]. Defines the party to be billed with the initial balance owed on the claim. Use SELF_PAY if you intend to bill self pay/cash pay.

            - provider_accepts_assignment: typing.Optional[bool]. Whether you have accepted the patient's authorization for insurance payments to be made to you, not them. Box 27 on the CMS-1500 claim form.

            - benefits_assigned_to_provider: typing.Optional[bool]. Whether this patient has authorized insurance payments to be made to you, not them. If false, patient may receive reimbursement. Box 13 on the CMS-1500 claim form.

            - synchronicity: typing.Optional[SynchronicityType]. Whether or not this was a synchronous or asynchronous encounter. Asynchronous encounters occur when providers and patients communicate online using forms, instant messaging, or other pre-recorded digital mediums. Synchronous encounters occur in live, real-time settings where the patient interacts directly with the provider, such as over video or a phone call.

            - place_of_service_code: typing.Optional[FacilityTypeCode]. Box 24B on the CMS-1500 claim form. Line-level place of service is not currently supported. 02 for telemedicine, 11 for in-person. Full list [here](https://www.cms.gov/Medicare/Coding/place-of-service-codes/Place_of_Service_Code_Set).

            - appointment_type: typing.Optional[str]. Human-readable description of the appointment type (ex: "Acupuncture - Headaches").

            - end_date_of_service: typing.Optional[Date]. Date formatted as YYYY-MM-DD; eg: 2019-08-25.
                                                          This date must be the local date in the timezone where the service occurred.
                                                          If omitted, the Encounter is assumed to be for a single day.
                                                          Must not be temporally before the date_of_service field.

            - subscriber_primary: typing.Optional[SubscriberCreate]. Contains details of the primary insurance subscriber.

            - subscriber_secondary: typing.Optional[SubscriberCreate]. Contains details of the secondary insurance subscriber.

            - additional_information: typing.Optional[str]. Defines additional information on the claim needed by the payer.
                                                            Box 19 on the CMS-1500 claim form.

            - service_authorization_exception_code: typing.Optional[ServiceAuthorizationExceptionCode]. 837p Loop2300 REF*4N
                                                                                                        Required when mandated by government law or regulation to obtain authorization for specific service(s) but, for the
                                                                                                        reasons listed in one of the enum values of ServiceAuthorizationExceptionCode, the service was performed without
                                                                                                        obtaining the authorization.

            - admission_date: typing.Optional[Date]. 837p Loop2300 DTP*435, CMS-1500 Box 18
                                                     Required on all ambulance claims when the patient was known to be admitted to the hospital.
                                                     OR
                                                     Required on all claims involving inpatient medical visits.

            - discharge_date: typing.Optional[Date]. 837p Loop2300 DTP*096, CMS-1500 Box 18
                                                     Required for inpatient claims when the patient was discharged from the facility and the discharge date is known.

            - onset_of_current_illness_or_symptom_date: typing.Optional[Date]. 837p Loop2300 DTP*431, CMS-1500 Box 14
                                                                               Required for the initial medical service or visit performed in response to a medical emergency when the date is available and is different than the date of service.
                                                                               OR
                                                                               This date is the onset of acute symptoms for the current illness or condition.

            - last_menstrual_period_date: typing.Optional[Date]. 837p Loop2300 DTP*484, CMS-1500 Box 14
                                                                 Required when, in the judgment of the provider, the services on this claim are related to the patient's pregnancy.

            - delay_reason_code: typing.Optional[DelayReasonCode]. 837i Loop2300, CLM-1300 Box 20
                                                                   Code indicating the reason why a request was delayed

        """
        _request: typing.Dict[str, typing.Any] = {}
        if prior_authorization_number is not OMIT:
            _request["prior_authorization_number"] = prior_authorization_number
        if external_id is not OMIT:
            _request["external_id"] = external_id
        if date_of_service is not OMIT:
            _request["date_of_service"] = date_of_service
        if diagnosis_ids is not OMIT:
            _request["diagnosis_ids"] = diagnosis_ids
        if tag_ids is not OMIT:
            _request["tag_ids"] = tag_ids
        if clinical_notes is not OMIT:
            _request["clinical_notes"] = clinical_notes
        if pay_to_address is not OMIT:
            _request["pay_to_address"] = pay_to_address
        if billable_status is not OMIT:
            _request["billable_status"] = billable_status
        if responsible_party is not OMIT:
            _request["responsible_party"] = responsible_party
        if provider_accepts_assignment is not OMIT:
            _request["provider_accepts_assignment"] = provider_accepts_assignment
        if benefits_assigned_to_provider is not OMIT:
            _request["benefits_assigned_to_provider"] = benefits_assigned_to_provider
        if synchronicity is not OMIT:
            _request["synchronicity"] = synchronicity
        if place_of_service_code is not OMIT:
            _request["place_of_service_code"] = place_of_service_code
        if appointment_type is not OMIT:
            _request["appointment_type"] = appointment_type
        if end_date_of_service is not OMIT:
            _request["end_date_of_service"] = end_date_of_service
        if subscriber_primary is not OMIT:
            _request["subscriber_primary"] = subscriber_primary
        if subscriber_secondary is not OMIT:
            _request["subscriber_secondary"] = subscriber_secondary
        if additional_information is not OMIT:
            _request["additional_information"] = additional_information
        if service_authorization_exception_code is not OMIT:
            _request["service_authorization_exception_code"] = service_authorization_exception_code
        if admission_date is not OMIT:
            _request["admission_date"] = admission_date
        if discharge_date is not OMIT:
            _request["discharge_date"] = discharge_date
        if onset_of_current_illness_or_symptom_date is not OMIT:
            _request["onset_of_current_illness_or_symptom_date"] = onset_of_current_illness_or_symptom_date
        if last_menstrual_period_date is not OMIT:
            _request["last_menstrual_period_date"] = last_menstrual_period_date
        if delay_reason_code is not OMIT:
            _request["delay_reason_code"] = delay_reason_code
        _response = await self._client_wrapper.httpx_client.request(
            "PATCH",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", f"api/encounters/v4/{encounter_id}"),
            json=jsonable_encoder(_request),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(Encounter, _response_json)  # type: ignore
        if "errorName" in _response_json:
            if _response_json["errorName"] == "EncounterExternalIdUniquenessError":
                raise EncounterExternalIdUniquenessError(
                    pydantic.parse_obj_as(EncounterExternalIdUniquenessErrorType, _response_json["content"])  # type: ignore
                )
            if _response_json["errorName"] == "EntityNotFoundError":
                raise EntityNotFoundError(
                    pydantic.parse_obj_as(EntityNotFoundErrorMessage, _response_json["content"])  # type: ignore
                )
            if _response_json["errorName"] == "UnauthorizedError":
                raise UnauthorizedError(
                    pydantic.parse_obj_as(UnauthorizedErrorMessage, _response_json["content"])  # type: ignore
                )
            if _response_json["errorName"] == "HttpRequestValidationsError":
                raise HttpRequestValidationsError(
                    pydantic.parse_obj_as(typing.List[RequestValidationError], _response_json["content"])  # type: ignore
                )
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class EncounterOwnerOfNextActionType(str, enum.Enum):
    CANDID = "CANDID"
    CUSTOMER = "CUSTOMER"
    CODER = "CODER"
    NONE = "NONE"

    def visit(
        self,
        candid: typing.Callable[[], T_Result],
        customer: typing.Callable[[], T_Result],
        coder: typing.Callable[[], T_Result],
        none: typing.Callable[[], T_Result],
    ) -> T_Result:
        if self is EncounterOwnerOfNextActionType.CANDID:
            return candid()
        if self is EncounterOwnerOfNextActionType.CUSTOMER:
            return customer()
        if self is EncounterOwnerOfNextActionType.CODER:
            return coder()
        if self is EncounterOwnerOfNextActionType.NONE:
            return none()



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

import typing_extensions

from .insurance_write_off_create import InsuranceWriteOffCreate
from .patient_write_off_create import PatientWriteOffCreate


class WriteOffCreate_Patient(PatientWriteOffCreate):
    type: typing_extensions.Literal["patient"]

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True


class WriteOffCreate_Insurance(InsuranceWriteOffCreate):
    type: typing_extensions.Literal["insurance"]

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True


WriteOffCreate = typing.Union[WriteOffCreate_Patient, WriteOffCreate_Insurance]



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

import typing_extensions

from .....commons.types.insurance_type_code import InsuranceTypeCode

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class InsuranceTypeCodes_InsuranceTypeCode(pydantic.BaseModel):
    type: typing_extensions.Literal["insurance_type_code"]
    value: InsuranceTypeCode

    class Config:
        frozen = True
        smart_union = True


class InsuranceTypeCodes_UnknownInsuranceTypeCode(pydantic.BaseModel):
    type: typing_extensions.Literal["unknown_insurance_type_code"]

    class Config:
        frozen = True
        smart_union = True


class InsuranceTypeCodes_NotApplicable(pydantic.BaseModel):
    type: typing_extensions.Literal["not_applicable"]

    class Config:
        frozen = True
        smart_union = True


InsuranceTypeCodes = typing.Union[
    InsuranceTypeCodes_InsuranceTypeCode, InsuranceTypeCodes_UnknownInsuranceTypeCode, InsuranceTypeCodes_NotApplicable
]



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

import typing_extensions

from .medicaid_provider_identifier import MedicaidProviderIdentifier
from .medicare_provider_identifier import MedicareProviderIdentifier


class IdentifierValue_MedicareProviderIdentifier(MedicareProviderIdentifier):
    type: typing_extensions.Literal["medicare_provider_identifier"]

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True


class IdentifierValue_MedicaidProviderIdentifier(MedicaidProviderIdentifier):
    type: typing_extensions.Literal["medicaid_provider_identifier"]

    class Config:
        frozen = True
        smart_union = True
        allow_population_by_field_name = True


IdentifierValue = typing.Union[IdentifierValue_MedicareProviderIdentifier, IdentifierValue_MedicaidProviderIdentifier]



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ......core.datetime_utils import serialize_datetime
from .....commons.types.claim_submission_payer_responsibility_type import ClaimSubmissionPayerResponsibilityType
from .claim_frequency_type_code import ClaimFrequencyTypeCode

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class ClaimSubmissionRecordCreate(pydantic.BaseModel):
    """
    Data about each external submission.
    ---
    import datetime

    from candid import ClaimSubmissionPayerResponsibilityType
    from candid.resources.claim_submission.v_1 import (ClaimFrequencyTypeCode,
                                                       ClaimSubmissionRecordCreate)

    ClaimSubmissionRecordCreate(submitted_at=datetime.datetime.fromisoformat("2023-01-01 13:00:00+00:00", ), claim_frequency_code=ClaimFrequencyTypeCode.1, payer_responsibility=ClaimSubmissionPayerResponsibilityType.PRIMARY, )
    """

    submitted_at: dt.datetime = pydantic.Field(description="When the claim was submitted to the payer.")
    claim_frequency_code: typing.Optional[ClaimFrequencyTypeCode] = None
    payer_responsibility: typing.Optional[ClaimSubmissionPayerResponsibilityType] = None

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        smart_union = True
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import typing
import urllib.parse
from json.decoder import JSONDecodeError

from .....core.api_error import ApiError
from .....core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from .....core.remove_none_from_dict import remove_none_from_dict
from ....commons.types.page_token import PageToken
from .types.payer import Payer
from .types.payer_page import PayerPage
from .types.payer_uuid import PayerUuid

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore


class V3Client:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def get(self, payer_uuid: PayerUuid) -> Payer:
        """
        Parameters:
            - payer_uuid: PayerUuid.
        """
        _response = self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", f"api/payers/v3/{payer_uuid}"),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(Payer, _response_json)  # type: ignore
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get_all(
        self,
        *,
        limit: typing.Optional[int] = None,
        search_term: typing.Optional[str] = None,
        page_token: typing.Optional[PageToken] = None,
    ) -> PayerPage:
        """
        Parameters:
            - limit: typing.Optional[int]. Maximum number of entities per page, defaults to 100.

            - search_term: typing.Optional[str].

            - page_token: typing.Optional[PageToken].
        ---
        from candid.client import CandidApi

        client = CandidApi(
            token="YOUR_TOKEN",
        )
        client.payers.v_3.get_all(
            limit=100,
            search_term="john",
            page_token="eyJ0b2tlbiI6IjEiLCJwYWdlX3Rva2VuIjoiMiJ9",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "api/payers/v3"),
            params=remove_none_from_dict({"limit": limit, "search_term": search_term, "page_token": page_token}),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(PayerPage, _response_json)  # type: ignore
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncV3Client:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def get(self, payer_uuid: PayerUuid) -> Payer:
        """
        Parameters:
            - payer_uuid: PayerUuid.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", f"api/payers/v3/{payer_uuid}"),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(Payer, _response_json)  # type: ignore
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get_all(
        self,
        *,
        limit: typing.Optional[int] = None,
        search_term: typing.Optional[str] = None,
        page_token: typing.Optional[PageToken] = None,
    ) -> PayerPage:
        """
        Parameters:
            - limit: typing.Optional[int]. Maximum number of entities per page, defaults to 100.

            - search_term: typing.Optional[str].

            - page_token: typing.Optional[PageToken].
        ---
        from candid.client import AsyncCandidApi

        client = AsyncCandidApi(
            token="YOUR_TOKEN",
        )
        await client.payers.v_3.get_all(
            limit=100,
            search_term="john",
            page_token="eyJ0b2tlbiI6IjEiLCJwYWdlX3Rva2VuIjoiMiJ9",
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "api/payers/v3"),
            params=remove_none_from_dict({"limit": limit, "search_term": search_term, "page_token": page_token}),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(PayerPage, _response_json)  # type: ignore
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

from ....core.api_error import ApiError
from ..types.organization_not_authorized_error_message import OrganizationNotAuthorizedErrorMessage


class OrganizationNotAuthorizedError(ApiError):
    def __init__(self, body: OrganizationNotAuthorizedErrorMessage):
        super().__init__(status_code=400, body=body)



# This file was auto-generated by Fern from our API Definition.

import typing
import urllib.parse
from json.decoder import JSONDecodeError

from .....core.api_error import ApiError
from .....core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from .....core.jsonable_encoder import jsonable_encoder
from .....core.remove_none_from_dict import remove_none_from_dict
from ....commons.errors.entity_not_found_error import EntityNotFoundError
from ....commons.errors.http_request_validation_error import HttpRequestValidationError
from ....commons.types.entity_not_found_error_message import EntityNotFoundErrorMessage
from ....commons.types.page_token import PageToken
from ....commons.types.request_validation_error import RequestValidationError
from .types.organization_service_facility import OrganizationServiceFacility
from .types.organization_service_facility_create import OrganizationServiceFacilityCreate
from .types.organization_service_facility_id import OrganizationServiceFacilityId
from .types.organization_service_facility_page import OrganizationServiceFacilityPage
from .types.organization_service_facility_update import OrganizationServiceFacilityUpdate

try:
    import pydantic.v1 as pydantic  # type: ignore
except ImportError:
    import pydantic  # type: ignore

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class V2Client:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def get(self, organization_service_facility_id: OrganizationServiceFacilityId) -> OrganizationServiceFacility:
        """
        Parameters:
            - organization_service_facility_id: OrganizationServiceFacilityId.
        """
        _response = self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/",
                f"api/organization-service-facilities/v2/{organization_service_facility_id}",
            ),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(OrganizationServiceFacility, _response_json)  # type: ignore
        if "errorName" in _response_json:
            if _response_json["errorName"] == "EntityNotFoundError":
                raise EntityNotFoundError(
                    pydantic.parse_obj_as(EntityNotFoundErrorMessage, _response_json["content"])  # type: ignore
                )
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get_multi(
        self,
        *,
        limit: typing.Optional[int] = None,
        name: typing.Optional[str] = None,
        page_token: typing.Optional[PageToken] = None,
    ) -> OrganizationServiceFacilityPage:
        """
        Parameters:
            - limit: typing.Optional[int]. Limit the number of results returned. Defaults to 100.

            - name: typing.Optional[str]. Filter to a name or a part of a name.

            - page_token: typing.Optional[PageToken]. The page token to continue paging through a previous request.
        ---
        from candid.client import CandidApi

        client = CandidApi(
            token="YOUR_TOKEN",
        )
        client.organization_service_facilities.v_2.get_multi(
            limit=100,
            name="Test Service Facility",
            page_token="eyJ0b2tlbiI6IjEiLCJwYWdlX3Rva2VuIjoiMiJ9",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "api/organization-service-facilities/v2"),
            params=remove_none_from_dict({"limit": limit, "name": name, "page_token": page_token}),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(OrganizationServiceFacilityPage, _response_json)  # type: ignore
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def create(self, *, request: OrganizationServiceFacilityCreate) -> OrganizationServiceFacility:
        """
        Parameters:
            - request: OrganizationServiceFacilityCreate.
        ---
        from candid import State, StreetAddressLongZip
        from candid.client import CandidApi
        from candid.resources.organization_service_facilities.v_2 import (
            OrganizationServiceFacilityCreate,
            ServiceFacilityMode,
            ServiceFacilityOperationalStatus,
            ServiceFacilityPhysicalType,
            ServiceFacilityStatus,
            ServiceFacilityType,
        )

        client = CandidApi(
            token="YOUR_TOKEN",
        )
        client.organization_service_facilities.v_2.create(
            request=OrganizationServiceFacilityCreate(
                name="Test Service Facility",
                aliases=["Test Service Facility Alias"],
                description="Test Service Facility Description",
                status=ServiceFacilityStatus.ACTIVE,
                operational_status=ServiceFacilityOperationalStatus.C,
                mode=ServiceFacilityMode.INSTANCE,
                type=ServiceFacilityType.DX,
                physical_type=ServiceFacilityPhysicalType.SI,
                telecoms=["555-555-5555"],
                address=StreetAddressLongZip(
                    address_1="123 Main St",
                    address_2="Apt 1",
                    city="New York",
                    state=State.NY,
                    zip_code="10001",
                    zip_plus_four_code="1234",
                ),
            ),
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "api/organization-service-facilities/v2"),
            json=jsonable_encoder(request),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(OrganizationServiceFacility, _response_json)  # type: ignore
        if "errorName" in _response_json:
            if _response_json["errorName"] == "HttpRequestValidationError":
                raise HttpRequestValidationError(
                    pydantic.parse_obj_as(RequestValidationError, _response_json["content"])  # type: ignore
                )
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def update(
        self,
        organization_service_facility_id: OrganizationServiceFacilityId,
        *,
        request: OrganizationServiceFacilityUpdate,
    ) -> OrganizationServiceFacility:
        """
        Parameters:
            - organization_service_facility_id: OrganizationServiceFacilityId.

            - request: OrganizationServiceFacilityUpdate.
        ---
        import uuid

        from candid import State, StreetAddressLongZip
        from candid.client import CandidApi
        from candid.resources.organization_service_facilities.v_2 import (
            OrganizationServiceFacilityUpdate,
            ServiceFacilityMode,
            ServiceFacilityOperationalStatus,
            ServiceFacilityPhysicalType,
            ServiceFacilityStatus,
            ServiceFacilityType,
        )

        client = CandidApi(
            token="YOUR_TOKEN",
        )
        client.organization_service_facilities.v_2.update(
            organization_service_facility_id=uuid.UUID(
                "30f55ee6-8c0e-43fc-a7fc-dac00d5bf569",
            ),
            request=OrganizationServiceFacilityUpdate(
                name="Test Service Facility",
                aliases=["Test Service Facility Alias"],
                description="Test Service Facility Description",
                status=ServiceFacilityStatus.ACTIVE,
                operational_status=ServiceFacilityOperationalStatus.C,
                mode=ServiceFacilityMode.INSTANCE,
                type=ServiceFacilityType.DX,
                physical_type=ServiceFacilityPhysicalType.SI,
                telecoms=["555-555-5555"],
                address=StreetAddressLongZip(
                    address_1="123 Main St",
                    address_2="Apt 1",
                    city="New York",
                    state=State.NY,
                    zip_code="10001",
                    zip_plus_four_code="1234",
                ),
            ),
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "PATCH",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/",
                f"api/organization-service-facilities/v2/{organization_service_facility_id}",
            ),
            json=jsonable_encoder(request),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(OrganizationServiceFacility, _response_json)  # type: ignore
        if "errorName" in _response_json:
            if _response_json["errorName"] == "HttpRequestValidationError":
                raise HttpRequestValidationError(
                    pydantic.parse_obj_as(RequestValidationError, _response_json["content"])  # type: ignore
                )
            if _response_json["errorName"] == "EntityNotFoundError":
                raise EntityNotFoundError(
                    pydantic.parse_obj_as(EntityNotFoundErrorMessage, _response_json["content"])  # type: ignore
                )
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def delete(self, organization_service_facility_id: OrganizationServiceFacilityId) -> None:
        """
        Parameters:
            - organization_service_facility_id: OrganizationServiceFacilityId.
        ---
        import uuid

        from candid.client import CandidApi

        client = CandidApi(
            token="YOUR_TOKEN",
        )
        client.organization_service_facilities.v_2.delete(
            organization_service_facility_id=uuid.UUID(
                "30f55ee6-8c0e-43fc-a7fc-dac00d5bf569",
            ),
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "DELETE",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/",
                f"api/organization-service-facilities/v2/{organization_service_facility_id}",
            ),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if "errorName" in _response_json:
            if _response_json["errorName"] == "EntityNotFoundError":
                raise EntityNotFoundError(
                    pydantic.parse_obj_as(EntityNotFoundErrorMessage, _response_json["content"])  # type: ignore
                )
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncV2Client:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def get(self, organization_service_facility_id: OrganizationServiceFacilityId) -> OrganizationServiceFacility:
        """
        Parameters:
            - organization_service_facility_id: OrganizationServiceFacilityId.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/",
                f"api/organization-service-facilities/v2/{organization_service_facility_id}",
            ),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(OrganizationServiceFacility, _response_json)  # type: ignore
        if "errorName" in _response_json:
            if _response_json["errorName"] == "EntityNotFoundError":
                raise EntityNotFoundError(
                    pydantic.parse_obj_as(EntityNotFoundErrorMessage, _response_json["content"])  # type: ignore
                )
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get_multi(
        self,
        *,
        limit: typing.Optional[int] = None,
        name: typing.Optional[str] = None,
        page_token: typing.Optional[PageToken] = None,
    ) -> OrganizationServiceFacilityPage:
        """
        Parameters:
            - limit: typing.Optional[int]. Limit the number of results returned. Defaults to 100.

            - name: typing.Optional[str]. Filter to a name or a part of a name.

            - page_token: typing.Optional[PageToken]. The page token to continue paging through a previous request.
        ---
        from candid.client import AsyncCandidApi

        client = AsyncCandidApi(
            token="YOUR_TOKEN",
        )
        await client.organization_service_facilities.v_2.get_multi(
            limit=100,
            name="Test Service Facility",
            page_token="eyJ0b2tlbiI6IjEiLCJwYWdlX3Rva2VuIjoiMiJ9",
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            "GET",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "api/organization-service-facilities/v2"),
            params=remove_none_from_dict({"limit": limit, "name": name, "page_token": page_token}),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(OrganizationServiceFacilityPage, _response_json)  # type: ignore
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def create(self, *, request: OrganizationServiceFacilityCreate) -> OrganizationServiceFacility:
        """
        Parameters:
            - request: OrganizationServiceFacilityCreate.
        ---
        from candid import State, StreetAddressLongZip
        from candid.client import AsyncCandidApi
        from candid.resources.organization_service_facilities.v_2 import (
            OrganizationServiceFacilityCreate,
            ServiceFacilityMode,
            ServiceFacilityOperationalStatus,
            ServiceFacilityPhysicalType,
            ServiceFacilityStatus,
            ServiceFacilityType,
        )

        client = AsyncCandidApi(
            token="YOUR_TOKEN",
        )
        await client.organization_service_facilities.v_2.create(
            request=OrganizationServiceFacilityCreate(
                name="Test Service Facility",
                aliases=["Test Service Facility Alias"],
                description="Test Service Facility Description",
                status=ServiceFacilityStatus.ACTIVE,
                operational_status=ServiceFacilityOperationalStatus.C,
                mode=ServiceFacilityMode.INSTANCE,
                type=ServiceFacilityType.DX,
                physical_type=ServiceFacilityPhysicalType.SI,
                telecoms=["555-555-5555"],
                address=StreetAddressLongZip(
                    address_1="123 Main St",
                    address_2="Apt 1",
                    city="New York",
                    state=State.NY,
                    zip_code="10001",
                    zip_plus_four_code="1234",
                ),
            ),
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            "POST",
            urllib.parse.urljoin(f"{self._client_wrapper.get_base_url()}/", "api/organization-service-facilities/v2"),
            json=jsonable_encoder(request),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(OrganizationServiceFacility, _response_json)  # type: ignore
        if "errorName" in _response_json:
            if _response_json["errorName"] == "HttpRequestValidationError":
                raise HttpRequestValidationError(
                    pydantic.parse_obj_as(RequestValidationError, _response_json["content"])  # type: ignore
                )
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def update(
        self,
        organization_service_facility_id: OrganizationServiceFacilityId,
        *,
        request: OrganizationServiceFacilityUpdate,
    ) -> OrganizationServiceFacility:
        """
        Parameters:
            - organization_service_facility_id: OrganizationServiceFacilityId.

            - request: OrganizationServiceFacilityUpdate.
        ---
        import uuid

        from candid import State, StreetAddressLongZip
        from candid.client import AsyncCandidApi
        from candid.resources.organization_service_facilities.v_2 import (
            OrganizationServiceFacilityUpdate,
            ServiceFacilityMode,
            ServiceFacilityOperationalStatus,
            ServiceFacilityPhysicalType,
            ServiceFacilityStatus,
            ServiceFacilityType,
        )

        client = AsyncCandidApi(
            token="YOUR_TOKEN",
        )
        await client.organization_service_facilities.v_2.update(
            organization_service_facility_id=uuid.UUID(
                "30f55ee6-8c0e-43fc-a7fc-dac00d5bf569",
            ),
            request=OrganizationServiceFacilityUpdate(
                name="Test Service Facility",
                aliases=["Test Service Facility Alias"],
                description="Test Service Facility Description",
                status=ServiceFacilityStatus.ACTIVE,
                operational_status=ServiceFacilityOperationalStatus.C,
                mode=ServiceFacilityMode.INSTANCE,
                type=ServiceFacilityType.DX,
                physical_type=ServiceFacilityPhysicalType.SI,
                telecoms=["555-555-5555"],
                address=StreetAddressLongZip(
                    address_1="123 Main St",
                    address_2="Apt 1",
                    city="New York",
                    state=State.NY,
                    zip_code="10001",
                    zip_plus_four_code="1234",
                ),
            ),
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            "PATCH",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/",
                f"api/organization-service-facilities/v2/{organization_service_facility_id}",
            ),
            json=jsonable_encoder(request),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(OrganizationServiceFacility, _response_json)  # type: ignore
        if "errorName" in _response_json:
            if _response_json["errorName"] == "HttpRequestValidationError":
                raise HttpRequestValidationError(
                    pydantic.parse_obj_as(RequestValidationError, _response_json["content"])  # type: ignore
                )
            if _response_json["errorName"] == "EntityNotFoundError":
                raise EntityNotFoundError(
                    pydantic.parse_obj_as(EntityNotFoundErrorMessage, _response_json["content"])  # type: ignore
                )
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def delete(self, organization_service_facility_id: OrganizationServiceFacilityId) -> None:
        """
        Parameters:
            - organization_service_facility_id: OrganizationServiceFacilityId.
        ---
        import uuid

        from candid.client import AsyncCandidApi

        client = AsyncCandidApi(
            token="YOUR_TOKEN",
        )
        await client.organization_service_facilities.v_2.delete(
            organization_service_facility_id=uuid.UUID(
                "30f55ee6-8c0e-43fc-a7fc-dac00d5bf569",
            ),
        )
        """
        _response = await self._client_wrapper.httpx_client.request(
            "DELETE",
            urllib.parse.urljoin(
                f"{self._client_wrapper.get_base_url()}/",
                f"api/organization-service-facilities/v2/{organization_service_facility_id}",
            ),
            headers=self._client_wrapper.get_headers(),
            timeout=60,
        )
        if 200 <= _response.status_code < 300:
            return
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if "errorName" in _response_json:
            if _response_json["errorName"] == "EntityNotFoundError":
                raise EntityNotFoundError(
                    pydantic.parse_obj_as(EntityNotFoundErrorMessage, _response_json["content"])  # type: ignore
                )
        raise ApiError(status_code=_response.status_code, body=_response_json)
